﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using ESRI.ArcGIS.Geoprocessing;
using ESRI.ArcGIS.esriSystem;
using ESRI.ArcGIS.Carto;
using ESRI.ArcGIS.Geodatabase;
using ESRI.ArcGIS.Framework;

namespace JAMBWOW
{
    public static class DDP_Util
    {        
        private const string ToolboxFileName = "DDPUtil.tbx";
        private const string ToolboxFolderName = "Python";

        private static int m_PageCount = -1;
        
        public static int CurrentKnownPage = 0;
        
        #region Public Methods

        #region Page Navigation Methods

        public static void GoToPage(int pageNum)
        {
            //Set the mouse cursor to an hourglass until this method finishes.
            IMouseCursor appCursor = new MouseCursorClass();
            appCursor.SetCursor(2);

            int return_value = 0;
            IGeoProcessor2 gp = GetGP();
            gp.OverwriteOutput = true;
            gp.ClearMessages();
            IVariantArray parameters = new VarArrayClass();
            parameters.Add("GoToPage");
            parameters.Add(pageNum);
            IGeoProcessorResult2 result;
            try
            {               
                //Run this method before the current page is changed by JambWow.
                PageChangeActions(pageNum, false);
                //Navigate to a new page.
                result = (IGeoProcessorResult2)gp.Execute("DDPUtil", parameters, null); //Execute the tool by name.
                return_value = Convert.ToInt32(result.ReturnValue);
                CurrentKnownPage = return_value;

            }
            catch (Exception e)
            {
                GenUtil.ShowError(e);
            }
            //ShowMessages(gp);
            ////Remove the custom Toolbox from ArcToolbox
            //RemoveToolbox(gp);
        }
        public static void FirstPage()
        {
            GoToPage(1);
        }
        public static void PrevPage()
        {
            int pageNum = GetCurrentPage() - 1;
            if (pageNum >= 1) GoToPage(pageNum);
        }
        public static void NextPage()
        {
            int pageCount = GetPageCount();
            int pageNum = GetCurrentPage() + 1;
            if (pageNum <= pageCount) GoToPage(pageNum);
        }
        public static void LastPage()
        {
            int pageCount = GetPageCount();
            GoToPage(pageCount);
        }

        #endregion

        #region Methods to get Current Page Index and Page Count

        public static int GetCurrentKnownPage()
        {
            return CurrentKnownPage;
        }
        public static int GetCurrentPage()
        {
            int return_value = 0;
            IGeoProcessor2 gp = GetGP();
            gp.OverwriteOutput = true;
            gp.ClearMessages();
            IVariantArray parameters = new VarArrayClass();
            parameters.Add("GetCurrentPage");
            IGeoProcessorResult2 result;
            try
            {
                result = (IGeoProcessorResult2)gp.Execute("DDPUtil", parameters, null); //Execute the tool by name.
                return_value = Convert.ToInt32(result.ReturnValue);
                CurrentKnownPage = return_value;
            }
            catch (Exception e)
            {
                GenUtil.ShowError(e);
            }
            //ShowMessages(gp); 
            ////Remove the custom Toolbox from ArcToolbox
            //RemoveToolbox(gp);
            return return_value;
        }

        public static int GetPageCount()
        { 
            return m_PageCount;
        }

        public static void InitializePageCount()
        {
            IGeoProcessor2 gp = GetGP();
            gp.OverwriteOutput = true;
            gp.ClearMessages();
            IVariantArray parameters = new VarArrayClass();
            parameters.Add("GetPageCount");
            IGeoProcessorResult2 result;
            try
            {
                result = (IGeoProcessorResult2)gp.Execute("DDPUtil", parameters, null); //Execute the tool by name.
                m_PageCount = Convert.ToInt32(result.ReturnValue);
            }
            catch (Exception e)
            {
                GenUtil.ShowError(e);
            }
            //ShowMessages(gp);
            ////Remove the custom Toolbox from ArcToolbox
            //RemoveToolbox(gp);
        }

        #endregion
        
        //This procedure runs every time the current page is changed by JambWow.
        public static void PageChangeActions(int pageIndex, bool bPartialRefresh)
        {
            LayoutElementTaggingUtil.UpdateTaggedElements(pageIndex, bPartialRefresh);
            HatchRotationUtil.UpdateHatchLabelRotation(pageIndex);
        }

        //This procedure returns true if data driven pages is activated.
        public static bool DDPActivated()
        {
            IPrintAndExport pPrintAndExport = new PrintAndExportClass();
            int PageCount = pPrintAndExport.get_PageCount(GenUtil.GetLayout());
            return (PageCount > 0);
        }

        #endregion

        #region private methods

        private static void ShowMessages(IGeoProcessor2 gp)
        {
            MessageBox.Show("Message Count = " + gp.MessageCount.ToString(), "JambWow");
            if (gp.MessageCount > 0)
            {
                for (int Count = 0; Count <= gp.MessageCount - 1; Count++)
                {
                    MessageBox.Show(gp.GetMessage(Count), "JambWow");
                }
            }
        }
        private static string GetToolboxPath()
        {
            return GenUtil.GetProgramPath() + @"\" + ToolboxFolderName;
        }

        private static IGeoProcessor2 GetGP()
        {
            IGeoProcessor2 gp = null;
            try
            {
                //Get the toolbox workspace.
                IWorkspaceFactory pToolboxWorkspaceFactory = new ToolboxWorkspaceFactoryClass();
                IToolboxWorkspace pToolboxWorkspace = (IToolboxWorkspace)pToolboxWorkspaceFactory.OpenFromFile(GetToolboxPath(), 0);

                //Add the toolbox to ArcToolbox??
                IGPToolbox pGPToolbox = pToolboxWorkspace.OpenToolbox(ToolboxFileName);

                //Initialize the geoprocessor.
                gp = new GeoProcessorClass();
                //Add the toolbox to the geoprocessing tool
                gp.AddToolbox(GetToolboxPath() + @"\" + ToolboxFileName);
            }
            catch (Exception e)
            {
                GenUtil.ShowError(e);
                gp = null;
            }
            return gp;
        }

        public static void RemoveToolbox()
        {
            IGeoProcessor2 gp = GetGP();
            gp.RemoveToolbox(GetToolboxPath() + @"\" + ToolboxFileName);                
        }

        //private static void RemoveToolbox(IGeoProcessor2 gp)
        //{
        //    //try
        //    //{
        //    //    //gp.RemoveToolbox(GetToolboxPath() + @"\" + ToolboxFileName);                
        //    //}
        //    //catch (Exception e)
        //    //{
        //    //    GenUtil.ShowError(e);
        //    //}
        //}

        #endregion
    }
}
