﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using ESRI.ArcGIS.Carto;
using ESRI.ArcGIS.esriSystem;
using ESRI.ArcGIS.Display;

namespace JAMBWOW
{
    public static class DataGraphUtil
    {
        public static void SetPropertiesOfDataGraph(IElementProperties pElemProps, int pageIndex)
        {
            if (!(pElemProps is IDataGraphTElement)) return; //If this is not a data graph element, then return.
            IDataGraphTElement pDataGraphtElement = (IDataGraphTElement)pElemProps;
            IDataGraphT dataGraph = pDataGraphtElement.DataGraphT; //Get the DataGraphT

            //Get the From Station and To Station values for the current page.
            bool bHasFromToStat = true;
            double FromStat;
            double ToStat;
            if (!double.TryParse(IndexLayerUtil.GetFromStation(pageIndex), out FromStat))
                bHasFromToStat = false;
            if (!double.TryParse(IndexLayerUtil.GetToStation(pageIndex), out ToStat))
                bHasFromToStat = false;
            if (!bHasFromToStat)
            {
                MessageBox.Show("Failed to update 'From Station' and 'To Station' of tagged datagraph", "JambWow");
                return;
            }

            //Get the minimum and maximum Y values for the current page, if these attributes exist.
            bool bHasMinMaxY = true;
            double MinY;
            double YRange;
            if (!double.TryParse(IndexLayerUtil.GetMinimumY(pageIndex), out MinY))
                bHasMinMaxY = false;
            if (!double.TryParse(IndexLayerUtil.GetYRange(pageIndex), out YRange))
                bHasMinMaxY = false;

            //If a minimum and maximum Y values exist,
            //modify the minimum and maximum values on the left and right axis.
            if (bHasMinMaxY)
            { 
                //Get the left and right axis
                IDataGraphTAxisProperties dataGraphAxisLeft = dataGraph.AxisProperties[0];
                IDataGraphTAxisProperties dataGraphAxisRight = dataGraph.AxisProperties[1];
                //Set the maximum Y
                double MaxY = MinY + YRange;                
                //Modify properties of the left and right axis.
                dataGraphAxisRight.AutomaticMaximum = false;
                dataGraphAxisRight.AutomaticMinimum = false;
                dataGraphAxisRight.Minimum = MinY;
                dataGraphAxisRight.Maximum = MaxY;
                dataGraphAxisLeft.AutomaticMaximum = false;
                dataGraphAxisLeft.AutomaticMinimum = false;
                dataGraphAxisLeft.Minimum = MinY;
                dataGraphAxisLeft.Maximum = MaxY;
            }

            //Modify the To and From station values on the bottom and top axis.
            //Get the top and bottom axis
            IDataGraphTAxisProperties dataGraphAxisBottom = dataGraph.AxisProperties[2];
            IDataGraphTAxisProperties dataGraphAxisTop = dataGraph.AxisProperties[3];
            //Modify properties of the top and bottom axis.

            //In order to avoid a bug in the ESRI DatagraphT Object,
            //we must set the minimum value to a dummy value that is GREATER THAN
            //the real minimum value that we want to set.  We first update then
            //graph with the dummy minimum value, then update again with the real
            //minumum value.  In this case, I chose to use the maximum value as my
            //dummy minimum value.
            dataGraphAxisBottom.AutomaticMaximum = false;
            dataGraphAxisBottom.AutomaticMinimum = false;
            dataGraphAxisTop.AutomaticMaximum = false;
            dataGraphAxisTop.AutomaticMinimum = false;
            dataGraphAxisBottom.Minimum = ToStat;
            dataGraphAxisBottom.Maximum = ToStat;
            dataGraphAxisTop.Minimum = ToStat;
            dataGraphAxisTop.Maximum = ToStat;                        
            //Update the data graph
            ITrackCancel pCancel = new CancelTrackerClass();
            pCancel.CancelOnClick = false;
            pCancel.CancelOnKeyPress = false;
            dataGraph.Update(pCancel);

            //Set the true minimum Station value for the top and bottom axis.
            dataGraphAxisBottom.Minimum = FromStat;           
            dataGraphAxisTop.Minimum = FromStat;            
            //Update the data graph
            ITrackCancel pCancel2 = new CancelTrackerClass();
            pCancel2.CancelOnClick = false;
            pCancel2.CancelOnKeyPress = false;
            dataGraph.Update(pCancel2);
        }
    }
}
