﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using ESRI.ArcGIS.OutputUI;
using ESRI.ArcGIS.Output;



namespace JAMBWOW
{
    public partial class FormExport : Form
    {
        [DllImport("user32.dll")]
        static extern IntPtr SetParent(IntPtr hWndChild, IntPtr hWndNewParent);

        [DllImport("user32.dll")]
        static extern bool ShowWindow(IntPtr hWnd, int nCmdShow);

        [DllImport("user32.dll")]
        static extern bool DestroyWindow(IntPtr hwnd);


        //Do I need a different path for 64-bit?
        //private const string USER_REG_PATH_PARAMS = @"Software\ESRI\Export\ExportObjectsParams" //9.3.1 path
        private const string USER_REG_PATH_PARAMS = @"Software\ESRI\Desktop10.0\Export\ExportObjectsParams"; //10.0 path

        private static FormExport m_ExportForm = null;

        public static FormExport ShowExportForm(PageRangeUtil.PageRangeTypes myPageRangeType, string sPageList = "")
        {
            try
            {
                if (m_ExportForm == null) m_ExportForm = new FormExport();

                //Select one of two Page Range type options.
                if (myPageRangeType == PageRangeUtil.PageRangeTypes.All)
                {
                    m_ExportForm.radAll.Select();
                    m_ExportForm.txtPages.Text = "";
                }
                if (myPageRangeType == PageRangeUtil.PageRangeTypes.PageList)
                {
                    m_ExportForm.radPages.Select();
                    m_ExportForm.txtPages.Text = sPageList;
                }
                //Show the export form
                m_ExportForm.Show(GenUtil.GetAppWinRef());
                //Return a pointer to the export form.
                return m_ExportForm;
            }
            catch (Exception pException)
            {
                GenUtil.ShowError(pException);
                return null;
            }
        }

        //Class-level instance variables
        private IExport m_pExport;
        private IExportPropertiesDialog m_pEPD;
      
        //Constructor
        private FormExport()
        {
            InitializeComponent();
            InitializeFileTypes();            
        }

        #region Methods tied to user interface events

        private void cmdOK_Click(object sender, EventArgs e)
        {
            try
            {
                m_ExportForm.Hide();

                //m_ExportForm.cmdOK.Enabled = false;
                //m_ExportForm.Cursor = Cursors.WaitCursor;


                ExportParameters pEP = GetGUIParameters();
                if (pEP == null) return;
                SaveExporterOptionChanges(); //Remember the user's choices for next time.
                
                
                ExportUtil.ExportPages(m_pExport, pEP);
                
                
                //m_ExportForm.Cursor = Cursors.Default;
                //m_ExportForm.cmdOK.Enabled = true;
                
                
                m_ExportForm.Close();
            }
            catch (Exception pException)
            {
                GenUtil.ShowError(pException);
            }
        }

        private void cmdCancel_Click(object sender, EventArgs e)
        {
            try
            {
                ResetExporterOptionChanges();
                m_ExportForm.Close();
            }
            catch (Exception pException)
            {
                GenUtil.ShowError(pException);
            }
        }

        private void cmdBrowse_Click(object sender, EventArgs e)
        {
            try
            {
                SaveFileDialog SFD = m_ExportForm.saveFileDialog1;
                SFD.Title = "Select a base file name (without page number or file extension).";
                if (SFD.ShowDialog() == DialogResult.OK)
                {
                    m_ExportForm.txtExportTo.Text = SFD.FileName;
                }
            }
            catch (Exception pException)
            {
                GenUtil.ShowError(pException);
            }
        }

        private void FormExport_FormClosing(object sender, FormClosingEventArgs e)
        {
            try
            {
                if (e.CloseReason == CloseReason.UserClosing) //If the user closed the form, hide it instead.
                {
                    e.Cancel = true;
                    this.Hide();
                }
                else //If the system closed the form, close it for real.
                {
                    if (m_pEPD != null) DestroyDialog(); //Discard the options dialog for previous file type
                    m_ExportForm = null;
                }
            }
            catch (Exception pException)
            {
                GenUtil.ShowError(pException);
            }
        }

        private void cmbFileType_SelectedIndexChanged(object sender, EventArgs e)
        {
            try
            {
                //Discard changes to user options for previous file type.
                if (m_pExport != null) ResetExporterOptionChanges();
                //Discard the options dialog for previous file type
                if (m_pEPD != null) DestroyDialog();

                //Initialize the UI based on the selected file type
                RefreshExporterOptions();
            }
            catch (Exception pException)
            {
                GenUtil.ShowError(pException);
            }
        }

        #endregion

        #region Non-UI methods

        private void InitializeFileTypes()
        {
            this.cmbFileType.Items.Clear();
            foreach (ExportFileType fileType in ExportParameters.FILE_TYPES)
            {
                this.cmbFileType.Items.Add(fileType);
            }
            //Select the first file type
            cmbFileType.SelectedIndex = 0; 
        }

        private ExportParameters GetGUIParameters()
        {
            ExportParameters pEP = new ExportParameters();
            //Get the base file export name
            if (txtExportTo.Text == "")
            {
                MessageBox.Show("You must select a base file name.", "JambWow");
                return null;
            }
            else pEP.BaseFileName = txtExportTo.Text;            
            //Get the file type
            if (cmbFileType.SelectedIndex == -1)
            {
                MessageBox.Show("You must select a file type.", "JambWow");
                return null;
            }
            else pEP.FileType = (ExportFileType) cmbFileType.SelectedItem;
            //Get page-ranges options.
            if (m_ExportForm.radAll.Checked) //All page
            {
                pEP.PageRangeType = PageRangeUtil.PageRangeTypes.All;
                pEP.PageList = "";
            }
            else if (m_ExportForm.radPages.Checked) //Select pages and page ranges.
            {
                pEP.PageRangeType = PageRangeUtil.PageRangeTypes.PageList;
                pEP.PageList = m_ExportForm.txtPages.Text;
            }
            //Output Disabled Pages? True/False
            pEP.OutputDisabledPages = !chkDontOutputDisabled.Checked;
            //Return the parameter object
            return pEP;
        }

        private void RefreshExporterOptions()
        {
            ExportFileType pFileType = (ExportFileType)cmbFileType.SelectedItem;
            string ExportType = pFileType.FileType;
            //MessageBox.Show("File Type: " + ExportType, "JambWow");

            //Initialize the exporter object based on the type of export file.
            m_pExport = ExportUtil.GetExportClass(ExportType);
            
            //Restore the exporter object's properties to the settings last saved in the registry.
            ResetExporterOptionChanges();

            //Initialize the "Export Properties Dialog" pointer to the proper dialog.
            m_pEPD = new DefaultExportPropertiesDialogClass();

            //Tell the "Export Properties Dialog" object to control the exporter object
            m_pEPD.SetObject(m_pExport);

            //Assign the HWnd of "Export Properties Dialog" to a widget on this Form.
            IntPtr my_hWnd = (IntPtr) m_pEPD.hWnd;
            SetParent(my_hWnd, panOptions.Handle);

            //Show the Export properties dialog inside a panel.
            ShowWindow(my_hWnd, 1);
        }

        private void ResetExporterOptionChanges()
        {
            ISettingsInRegistry pSIR = (ISettingsInRegistry)m_pExport;
            pSIR.RestoreForCurrentUser(USER_REG_PATH_PARAMS);
        }

        private void SaveExporterOptionChanges()
        {
            ISettingsInRegistry pSIR = (ISettingsInRegistry)m_pExport;
            pSIR.StoreForCurrentUser(USER_REG_PATH_PARAMS);            
        }

        private void DestroyDialog()
        {
            IntPtr my_hWnd = (IntPtr)m_pEPD.hWnd;
            DestroyWindow(my_hWnd);
        }

        #endregion

    }
}
