﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using ESRI.ArcGIS.ArcMapUI;
using ESRI.ArcGIS.Framework;
using ESRI.ArcGIS.Output;

namespace JAMBWOW
{
    public partial class FormPrint : Form
    {
        //Constructor
        public FormPrint(PageRangeUtil.PageRangeTypes myPageRangeType, string sPageList = "")
        {
            InitializeComponent();
            try
            {
                //Select one of two Page Range type options.
                if (myPageRangeType == PageRangeUtil.PageRangeTypes.All)
                {
                    radAll.Select();
                    txtPages.Text = "";
                }
                if (myPageRangeType == PageRangeUtil.PageRangeTypes.PageList)
                {
                    radPages.Select();
                    txtPages.Text = sPageList;
                }
                //Display properties of the current printer.
                DisplayPrinterProperties();
            }
            catch (Exception pException)
            {
                GenUtil.ShowError(pException);
            }
        }

        #region GUI Methods

        private void cmdSetup_Click(object sender, EventArgs e)
        {
            try
            {
                ShowPageSetupDialog();
            }
            catch (Exception pException)
            {
                GenUtil.ShowError(pException);
            }
        }

        private void cmdOk_Click(object sender, EventArgs e)
        {
            try
            {
                this.Close();
            }
            catch (Exception pException)
            {
                GenUtil.ShowError(pException);
            }
        }

        private void cmdCancel_Click(object sender, EventArgs e)
        {
            try
            {
                this.Close();
            }
            catch (Exception pException)
            {
                GenUtil.ShowError(pException);
            }
        }

        #endregion

        #region Non-GUI methods

        public PrintParameters GetGUIParameters()
        {
            PrintParameters pPP = new PrintParameters();
            //Get page-ranges options.
            if (radAll.Checked) //All page
            {
                pPP.PageRangeType = PageRangeUtil.PageRangeTypes.All;
                pPP.PageList = "";
            }
            else if (radPages.Checked) //Select pages and page ranges.
            {
                pPP.PageRangeType = PageRangeUtil.PageRangeTypes.PageList;
                pPP.PageList = txtPages.Text;
            }
            //If the user entered a non-numeric value for number of copies, return 1 instead.
            int iNumCopies;
            if (int.TryParse(this.txtCopies.Text, out iNumCopies))
                pPP.NumberOfCopies = iNumCopies;
            else
                pPP.NumberOfCopies = 1;
            
            //Output Disabled Pages? True/False
            pPP.OutputDisabledPages = !chkDontOutputDisabled.Checked;
            //Return the parameter object
            return pPP;
        }

        private void ShowPageSetupDialog()
        {
            IApplication pApp = GenUtil.GetAppRef();
            IMxApplication pMxApp = (IMxApplication)pApp;
            if (!pApp.IsDialogVisible((int)esriMxDlgIDs.esriMxDlgPageSetup))
            {
                //Display the page setup dialog.
                object bShow = (object)true;
                pApp.ShowDialog((int)esriMxDlgIDs.esriMxDlgPageSetup, bShow);

                //Wait for the dialog to close.
                bool bDialog = true;
                while (bDialog)
                {
                    bDialog = pApp.IsDialogVisible((int)esriMxDlgIDs.esriMxDlgPageSetup);
                    System.Windows.Forms.Application.DoEvents(); //Allow other processes to run.
                    System.Threading.Thread.Sleep(1);
                }
                //Display properties of the current printer.
                DisplayPrinterProperties();
            }
        }

        //Display properties of the current printer.
        private void DisplayPrinterProperties()
        {
            IApplication pApp = GenUtil.GetAppRef();
            IMxApplication pMxApp = (IMxApplication)pApp;            
            IPrinter pPrinter = pMxApp.Printer;
            this.lblName.Text = pPrinter.Paper.PrinterName;
            this.lblType.Text = pPrinter.DriverName;
        }

        #endregion
    }
}
