﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using ESRI.ArcGIS.Carto;
using ESRI.ArcGIS.esriSystem;

namespace JAMBWOW
{
    public partial class FormSelectedPages : Form
    {
        //Constructor
        public FormSelectedPages(List<PageWrapper> HiddenOnPages, List<PageWrapper> NotHiddenOnPages)
        {
            InitializeComponent();
            //Convert the lists of PageWrappers to lists of Objects.
            List<object> HiddenOnPagesObj = new List<object>();
            List<object> NotHiddenOnPagesObj = new List<object>();
            foreach (PageWrapper p in HiddenOnPages) HiddenOnPagesObj.Add(p);
            foreach (PageWrapper p in NotHiddenOnPages) NotHiddenOnPagesObj.Add(p);
            //Initialize the shuttle lists.
            shuttleListPages.InitializeLists(NotHiddenOnPagesObj, HiddenOnPagesObj);
        }

        #region Methods tied to user interface events

        private void cmdOK_Click(object sender, EventArgs e)
        {
            //Validate that Item will be shown on at least one page.
            if (shuttleListPages.lstMoveFrom.Items.Count == 0)
            {
                MessageBox.Show("The page item must be visible on at least one page.", "JambWow");
                return;
            }
            this.Close();
        }

        private void cmdCancel_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        //This event is called as the form is closing.
        private void FormSelectedPages_FormClosing(object sender, FormClosingEventArgs e)
        {
        }

        #endregion

        #region Non-UI methods

        //This method returns a list of pages on which the page elements should be hidden.
        public List<string> GetPagesHiddenOn()
        {
            List<string> PagesHiddenOn = new List<string>();
            foreach (object MyObject in shuttleListPages.lstMoveTo.Items)
            {
                PageWrapper pPageWrapper = (PageWrapper)MyObject; //Cast the objects to PageWrapper objects.
                string sName = pPageWrapper.PageName;
                PagesHiddenOn.Add(sName);
            }
            return PagesHiddenOn;
        }

        #endregion
    }
}
