﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Drawing;
using System.Runtime.InteropServices;
using ESRI.ArcGIS.Framework;
using ESRI.ArcGIS.ArcMapUI;
using ESRI.ArcGIS.Carto;
using ESRI.ArcGIS.Output;
using ESRI.ArcGIS.esriSystem;
using ESRI.ArcGIS.Geometry;
using ESRI.ArcGIS.Display;

namespace JAMBWOW
{
    public static class GenUtil
    {
        public const string IFEATURE_LAYER_UID = "{40A9E885-5533-11d0-98BE-00805F7CED21}";
        public const string IANNOTATION_LAYER_UID = "{4AEDC069-B599-424B-A374-49602ABAD308}";
        public const string IGROUP_LAYER_UID = "{EDAD6644-1810-11D1-86AE-0000F8751720}";

        public const int DATA_FRAME_INDEX = 0;

        private static string m_AddInPath = "";

        //Get the path to the ArcGIS Install
        public static string GetArcGISInstallPath()
        {
            const string default_path = @"c:\Program Files\ArcGIS\Desktop10.0\";
            try
            {
                Microsoft.Win32.RegistryKey regKey = Microsoft.Win32.Registry.LocalMachine.OpenSubKey("SOFTWARE\\ESRI\\Desktop10.0", false);
                string sInstallDir = (string) regKey.GetValue("InstallDir", "");
                //Use this alternate approach on 64-bit machines.
                if (sInstallDir == "")
                {
                    regKey = Microsoft.Win32.Registry.LocalMachine.OpenSubKey("SOFTWARE\\WOW6432Node\\ESRI\\Desktop10.0", false);
                    sInstallDir = (string)regKey.GetValue("InstallDir", "");
                }
                //As a last resort, use the default path
                if (sInstallDir == "")
                    sInstallDir = default_path;
                return sInstallDir;
            }
            catch (Exception pException)
            {
                GenUtil.ShowError(pException);
                return default_path;
            }
        }

        //Get the path to AddIn .dll and other file content.
        public static string GetProgramPath()
        {
            if (m_AddInPath == "") //Use lazy evaluation
            {
                m_AddInPath = System.IO.Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().Location);
            }
            return m_AddInPath;
        }

        //Get a reference to the current application object
        public static IApplication GetAppRef()
        {
            return ArcMap.Application;        
        }

        //Get the window handle of the current application object
        public static IWin32Window GetAppWinRef()
        {
            return NativeWindow.FromHandle(new IntPtr(ArcMap.Application.hWnd));
        }


        public static IMxDocument GetMxDocument()
        {
            return ArcMap.Document;
        }

        public static IMap GetFirstDataFrame()
        {
            return ArcMap.Document.Maps.get_Item(GenUtil.DATA_FRAME_INDEX);
        }

        public static IActiveView GetLayout()
        {
            return (IActiveView)ArcMap.Document.PageLayout;
        }

        public static string GetMapDocumentFileName()
        {
            return ((IMapDocument)ArcMap.Document).DocumentFilename;
        }
        
        ////Hide the Data Driven Pages Toolbar
        //public static void HideDataDrivenPagesToolbar()
        //{            
        //    GenUtil.HideCommandBar("esriArcMapUI.PageIndexToolBar");
        //}

        ////Hide a command bar
        //public static void HideCommandBar(string CommandBar_ProgID)
        //{
        //        //Get a UID for the CommandBar
        //        UID CommandBar_UID = new UID();
        //        CommandBar_UID.Value = CommandBar_ProgID;
        //        //Get a colletion of command bars
        //        IDocument ThisDoc = ArcMap.Application.Document;
        //        ICommandBars CommandBars = ThisDoc.CommandBars as ICommandBars;
        //        //Get the specific command bar by UID
        //        ICommandBar MyCommandBar = CommandBars.Find(CommandBar_UID, false, false) as ICommandBar;
        //        ICommandItem pCommandItem = (ICommandItem)MyCommandBar;
        //        pCommandItem.Delete();
        //}

        //Returns the path to the assembly
        static public string AssemblyDirectory
        {
            get
            {
                string codeBase = System.Reflection.Assembly.GetExecutingAssembly().CodeBase;
                UriBuilder uri = new UriBuilder(codeBase);
                string path = Uri.UnescapeDataString(uri.Path);
                return System.IO.Path.GetDirectoryName(path);
            }
        }

        //This function creates a file and writes the contents of a string into it.
        //This function has been tested with a file containing 680,000 characters.
        public static void WriteToFile(string FilePath, string contents)
        {
            StreamWriter SW = new StreamWriter(FilePath, false, System.Text.Encoding.Default);
            SW.Write(contents);
            SW.Close();
        }

        //Show an error messagebox
        public static void ShowError(Exception pException)
        {
            if (pException is ExceptionUserError)
            {
                //MessageBox.Show(pException.Message, "Error: Problem is between chair and keyboard.");
                MessageBox.Show(pException.Message, "User Error:");                
            }
            else
            {
                MessageBox.Show("Oops, that wasn't supposed to happen. "
                    + Environment.NewLine + Environment.NewLine
                    + pException.ToString()
                    + Environment.NewLine + Environment.NewLine
                    + "Press Ctrl+C to copy this error message.",
                    "An unexpected error has occurred in JambWow.");
            }
        }
    }
}
