﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using ESRI.ArcGIS.Carto;
using ESRI.ArcGIS.esriSystem;
using ESRI.ArcGIS.Location;
using ESRI.ArcGIS.Display;

namespace JAMBWOW
{
    public static class HatchRotationUtil
    {
        //This procedure finds all of the feature layers in the map that have hatches
        //and rotates the hatch labels to PageRoation * -1.        
        //If the index layer does not have a rotation attribute, do nothing.
        public static void UpdateHatchLabelRotation(int pageIndex)
        {
            double dPageRotation;
            if (!double.TryParse(IndexLayerUtil.GetRotation(pageIndex), out dPageRotation)) return;
            double dHatchRotation = dPageRotation * -1;
            IMap pMap = GenUtil.GetFirstDataFrame();

            //Get an enumeration of feature layers in the data frame
            UID uid = new UIDClass();
            uid.Value = GenUtil.IFEATURE_LAYER_UID;
            try
            {
                IEnumLayer pEnumLayers = pMap.get_Layers(uid, true);
                //Loop through the layers
                pEnumLayers.Reset();
                ILayer pLayer = null;
                while ((pLayer = pEnumLayers.Next()) != null)
                {
                    ILayerExtensions pLayerExt = (ILayerExtensions) pLayer;                    
                    //Loop through each of the "layer extensions" to try and find a "Hatch Layer" extension.
                    if (pLayerExt.ExtensionCount > 0)
                    {
                        for (int i = 0; i < pLayerExt.ExtensionCount; i++)
                        {
                            if (pLayerExt.get_Extension(i) is IHatchLayerExtension)
                            {
                                IHatchLayerExtension pHatchLayerExt = (IHatchLayerExtension)pLayerExt.get_Extension(i);
                                
                                //Loop through each hatch class in the IHatchLayerExtension
                                for (int x = 0; x < pHatchLayerExt.HatchClassCount(); x++)
                                {
                                    string[] HatchClassNames = (string[]) pHatchLayerExt.HatchClassNames();
                                    IHatchClass pHatchClass = pHatchLayerExt.HatchClass(HatchClassNames[x]);

                                    //Get the Hatch Definition enumeration
                                    IHatchTemplate pHatchTemplate = pHatchClass.HatchTemplate;
                                    IEnumHatchDefinition pEnumHatchDefinition = pHatchTemplate.EnumHatchDefinitions;
                                    
                                    //Loop through the Hatch Definition enumeration
                                    int iHatchInterval = 0;
                                    IHatchDefinition pHatchDefinition = null;
                                    pEnumHatchDefinition.Reset();
                                    pEnumHatchDefinition.Next(ref iHatchInterval, ref pHatchDefinition);
                                    while (pHatchDefinition != null)
                                    {
                                        //Modify the hatch definition here by changing the angle of rotation of the text symbol.                                        
                                        //If labeling is turned on.
                                        if (pHatchDefinition.TextDisplay != esriHatchTextDisplay.esriHatchTDNone) 
                                        {
                                            ITextSymbol pTextSymbol = pHatchDefinition.TextSymbol;
                                            if (pTextSymbol != null)
                                            {
                                                pTextSymbol.Angle = dHatchRotation;
                                                pHatchDefinition.TextSymbol = pTextSymbol;
                                            }
                                        }
                                        //Increment the loop
                                        pEnumHatchDefinition.Next(ref iHatchInterval, ref pHatchDefinition);
                                    }

                                    //Now perform the same logic on the EndHatchDefinition
                                    pHatchDefinition = pHatchTemplate.EndHatchDefinition;
                                    if (pHatchDefinition != null)
                                    {
                                        //Modify the hatch definition here by changing the angle of rotation of the text symbol.                                        
                                        //If labeling is turned on.
                                        if (pHatchDefinition.TextDisplay != esriHatchTextDisplay.esriHatchTDNone)
                                        {
                                            ITextSymbol pTextSymbol = pHatchDefinition.TextSymbol;
                                            if (pTextSymbol != null)
                                            {
                                                pTextSymbol.Angle = dHatchRotation;
                                                pHatchDefinition.TextSymbol = pTextSymbol;
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
            catch (Exception e)
            {
                GenUtil.ShowError(e);
            }
        }
    }
}
