﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using ESRI.ArcGIS.Carto;
using ESRI.ArcGIS.Framework;

namespace JAMBWOW
{
    /// <summary>
    /// Designer class of the dockable window add-in. It contains user interfaces that
    /// make up the dockable window.
    /// </summary>
    public partial class JAMBWOW_DockableWindow : UserControl
    {
        //Static Variables
        private static JAMBWOW_DockableWindow m_JAMBWOW_DockableWindow = null;
        //Static Methods
        public static JAMBWOW_DockableWindow GetInstance()
        {
            return m_JAMBWOW_DockableWindow;
        }

        public JAMBWOW_DockableWindow(object hook)
        {
            InitializeComponent();
            this.Hook = hook;
        }

        /// <summary>
        /// Host object of the dockable window
        /// </summary>
        private object Hook
        {
            get;
            set;
        }

        /// <summary>
        /// Implementation class of the dockable window add-in. It is responsible for 
        /// creating and disposing the user interface class of the dockable window.
        /// </summary>
        public class AddinImpl : ESRI.ArcGIS.Desktop.AddIns.DockableWindow
        {
            private JAMBWOW_DockableWindow m_windowUI;

            public AddinImpl()
            {
            }

            protected override IntPtr OnCreateChild()
            {
                m_windowUI = new JAMBWOW_DockableWindow(this.Hook);
                m_JAMBWOW_DockableWindow = m_windowUI;
                return m_windowUI.Handle;
            }

            protected override void Dispose(bool disposing)
            {
                if (m_windowUI != null)
                    m_windowUI.Dispose(disposing);

                base.Dispose(disposing);
            }
        }

        #region Methods tied to user interface events

        #region Dockable Window Events

        private void JAMBWOW_DockableWindow_Load(object sender, EventArgs e)
        {
            //try
            //{
            //    GenUtil.HideDataDrivenPagesToolbar();
            //}
            //catch (Exception pException)
            //{
            //    GenUtil.ShowError(pException);
            //}
        }

        private void JAMBWOW_DockableWindow_VisibleChanged(object sender, EventArgs e)
        {
            //try
            //{
            //    if (this.Visible)
            //        GenUtil.HideDataDrivenPagesToolbar();
            //}
            //catch (Exception pException)
            //{
            //    GenUtil.ShowError(pException);
            //}
        }

        #endregion

        #region Tool Strip Events

        private void toolStripFirstPage_Click(object sender, EventArgs e)
        {
            try
            {
                DDP_Util.FirstPage();
                DisplayCurrentPageNumber();
            }
            catch (Exception pException)
            {
                GenUtil.ShowError(pException);
            }
        }

        private void toolStripPrevPage_Click(object sender, EventArgs e)
        {
            try
            {
                DDP_Util.PrevPage();
                DisplayCurrentPageNumber();
            }
            catch (Exception pException)
            {
                GenUtil.ShowError(pException);
            }
        }

        private void toolStripNextPage_Click(object sender, EventArgs e)
        {
            try
            {
                DDP_Util.NextPage();
                DisplayCurrentPageNumber();
            }
            catch (Exception pException)
            {
                GenUtil.ShowError(pException);
            }
        }

        private void toolStripLastPage_Click(object sender, EventArgs e)
        {
            try
            {
                DDP_Util.LastPage();
                DisplayCurrentPageNumber();
            }
            catch (Exception pException)
            {
                GenUtil.ShowError(pException);
            }
        }

        private void toolStripRefresh_Click(object sender, EventArgs e)
        {
            try
            {
                ActivateAndRefreshDialog();
            }
            catch (Exception pException)
            {
                GenUtil.ShowError(pException);
            }
        }

        #endregion

        #region TreeView Events

        private void treeViewPages_NodeMouseDoubleClick(object sender, TreeNodeMouseClickEventArgs e)
        {
            try
            {
                GoToSelectedPage(e.Node);
            }
            catch (Exception pException)
            {
                GenUtil.ShowError(pException);
            }
        }

        private void treeViewPages_NodeMouseClick(object sender, TreeNodeMouseClickEventArgs e)
        {
            try
            {
                ShowContextMenu(e);
            }
            catch (Exception pException)
            {
                GenUtil.ShowError(pException);
            }
        }

        private void treeViewPages_BeforeCheck(object sender, TreeViewCancelEventArgs e)
        {
            //Prevent the root node from being disabled.
            if (e.Node.Parent == null) e.Cancel = true;
        }

        private void treeViewPages_AfterCheck(object sender, TreeViewEventArgs e)
        {
            try
            {
                if (e.Node.Parent == null) return; //If this is the root node, return.
                bool bDisabled = e.Node.Checked;
                string sPageIndex = e.Node.Name;
                int iPageIndex;
                if(int.TryParse(sPageIndex, out iPageIndex))
                    UpdateEnabledDisabledPageStatus(iPageIndex, bDisabled);               
            }
            catch (Exception pException)
            {
                GenUtil.ShowError(pException);
            }
        }


        #endregion

        #region Page Context Menu Events

        private void toolStripMenuItemViewPage_Click(object sender, EventArgs e)
        {
            try
            {
                TreeNode myNode = treeViewPages.SelectedNode;
                GoToSelectedPage(myNode);
            }
            catch (Exception pException)
            {
                GenUtil.ShowError(pException);
            }
        }

        private void toolStripMenuItemPrintPage_Click(object sender, EventArgs e)
        {
            try
            {
                TreeNode myNode = treeViewPages.SelectedNode;
                int pageNum = Convert.ToInt32(myNode.Name);                
                //Show the Print Form.
                FormPrint pFormPrint = new FormPrint(PageRangeUtil.PageRangeTypes.PageList, pageNum.ToString());
                pFormPrint.ShowDialog(GenUtil.GetAppWinRef());

                //If the user closes the dialog by clicking "OK", print the select pages.
                if (pFormPrint.DialogResult == DialogResult.OK)
                {
                    PrintParameters pPP = pFormPrint.GetGUIParameters();
                    PrintUtil.PrintPages(pPP);
                }
            }
            catch (Exception pException)
            {
                GenUtil.ShowError(pException);
            }
        }

        private void toolStripMenuItemExportPage_Click(object sender, EventArgs e)
        {
            try
            {
                TreeNode myNode = treeViewPages.SelectedNode;
                int pageNum = Convert.ToInt32(myNode.Name);
                //Show the Export Form.
                FormExport.ShowExportForm(PageRangeUtil.PageRangeTypes.PageList, pageNum.ToString());
            }
            catch (Exception pException)
            {
                GenUtil.ShowError(pException);
            }
        }

        #endregion

        #region Map Series Context Menu Events
        
        private void toolStripMenuItemPrintSeries_Click(object sender, EventArgs e)
        {
            try
            {
                //Show the Print Form.
                FormPrint pFormPrint = new FormPrint(PageRangeUtil.PageRangeTypes.All);                
                pFormPrint.ShowDialog(GenUtil.GetAppWinRef());

                //If the user closes the dialog by clicking "OK", print the select pages.
                if (pFormPrint.DialogResult == DialogResult.OK)
                {
                    PrintParameters pPP = pFormPrint.GetGUIParameters();
                    PrintUtil.PrintPages(pPP);
                }
            }
            catch (Exception pException)
            {
                GenUtil.ShowError(pException);
            }
        }

        private void toolStripMenuItemExportSeries_Click(object sender, EventArgs e)
        {
            try
            {
                FormExport.ShowExportForm(PageRangeUtil.PageRangeTypes.All);
            }
            catch (Exception pException)
            {
                GenUtil.ShowError(pException);
            }
        }

        private void toolStripMenuItemConvertLabelsToAnnotation_Click(object sender, EventArgs e)
        {
            try
            {
                FormConvertLabelsToAnnotation.ShowAnnoForm();
            }
            catch (Exception pException)
            {
                GenUtil.ShowError(pException);
            }
        }


        private void toolStripMenuItemDeleteAnnotation_Click(object sender, EventArgs e)
        {
            try
            {
                FormDeleteAnnotation.ShowDeleteAnnoForm();
            }
            catch (Exception pException)
            {
                GenUtil.ShowError(pException);
            }
        }

        private void ConvertLabelsToAnnoThisPageOnly_Click(object sender, EventArgs e)
        {
            try
            {
                FormConvertLabelsToAnnotation.ShowAnnoForm(DDP_Util.GetCurrentKnownPage());
            }
            catch (Exception pException)
            {
                GenUtil.ShowError(pException);
            }
        }

        private void DeleteAnnotationThisPageOnly_Click(object sender, EventArgs e)
        {
            try
            {
                FormDeleteAnnotation.ShowDeleteAnnoForm(DDP_Util.GetCurrentKnownPage());
            }
            catch (Exception pException)
            {
                GenUtil.ShowError(pException);
            }
        }

        private void toolStripMenuItemTagDataGraph_Click(object sender, EventArgs e)
        {
            try
            {
                LayoutElementTaggingUtil.TagOrUntagDataGraph(true);
            }
            catch (Exception pException)
            {
                GenUtil.ShowError(pException);
            }
        }

        private void toolStripMenuItemUntagDataGraph_Click(object sender, EventArgs e)
        {
            try
            {
                LayoutElementTaggingUtil.TagOrUntagDataGraph(false);
            }
            catch (Exception pException)
            {
                GenUtil.ShowError(pException);
            }
        }

        private void toolStripMenuItemTagToShowOnThisPageOnly_Click(object sender, EventArgs e)
        {
            try
            {
                LayoutElementTaggingUtil.TagToShowOnThisPageOnly();
            }
            catch (Exception pException)
            {
                GenUtil.ShowError(pException);
            }
        }

        private void toolStripMenuItemTagToHideOnSelectPages_Click(object sender, EventArgs e)
        {
            try
            {
                LayoutElementTaggingUtil.TagToHideOnSelectedPages();                
            }
            catch (Exception pException)
            {
                GenUtil.ShowError(pException);
            }
        }

        private void toolStripMenuItemUntagToShowOnAllPages_Click(object sender, EventArgs e)
        {
            try
            {
                LayoutElementTaggingUtil.UntagToShowOnAllPages();
            }
            catch (Exception pException)
            {
                GenUtil.ShowError(pException);
            }
        }

        private void toolStripMenuItemTagStandardPositionAndSize_Click(object sender, EventArgs e)
        {
            try
            {
                LayoutElementTaggingUtil.TagOrUntagStandardGeometry(true);
            }
            catch (Exception pException)
            {
                GenUtil.ShowError(pException);
            }
        }

        private void toolStripMenuItemRemovePositonAndSizeTags_Click(object sender, EventArgs e)
        {
            try
            {
                LayoutElementTaggingUtil.TagOrUntagStandardGeometry(false); ;
            }
            catch (Exception pException)
            {
                GenUtil.ShowError(pException);
            }
        }

        private void toolStripMenuItemTagAlternatePositionAndSize_Click(object sender, EventArgs e)
        {
            try
            {
                LayoutElementTaggingUtil.TagOrUntagAlternateGeometry(true);
            }
            catch (Exception pException)
            {
                GenUtil.ShowError(pException);
            }
        }

        private void toolStripMenuItemApplyStandardToThisPage_Click(object sender, EventArgs e)
        {
            try
            {
                LayoutElementTaggingUtil.TagOrUntagAlternateGeometry(false);
            }
            catch (Exception pException)
            {
                GenUtil.ShowError(pException);
            }
        }


        private void upgradeOldJambAnnotationToolStripMenuItem_Click(object sender, EventArgs e)
        {
            try
            {
                JambMapUpgrader.UpgradeMapAnnotationToGeodatabase();
            }
            catch (Exception pException)
            {
                GenUtil.ShowError(pException);
            }
        }

        private void upgradeOldJambTaggedLayoutElementsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            try
            {
                JambMapUpgrader.UpgradeLayoutElements();
            }
            catch (Exception pException)
            {
                GenUtil.ShowError(pException);
            }
        }


        #endregion

        #endregion

        #region Non-UI methods
     
        public void ClearDialog()
        {
            try
            {
                //Clear the treeview
                treeViewPages.Nodes.Clear();
                //Set the current page number label to nothing.
                lblCurrentPage.Text = "Current Page: ";
                //Disable the navigation buttons
                toolStripFirstPage.Enabled = false;
                toolStripPrevPage.Enabled = false;
                toolStripNextPage.Enabled = false;
                toolStripLastPage.Enabled = false;
            }
            catch (Exception e)
            {
                GenUtil.ShowError(e);
            }
        }

        public void ActivateAndRefreshDialog()
        {        
            if (!DDP_Util.DDPActivated())
            {
                MessageBox.Show("Data Driven Pages Are Not Activated. Activate DDP and try again.", "JambWow");
                return;
            }

            try
            {
                //Set the mouse cursor to an hourglass until this method finishes.
                IMouseCursor appCursor = new MouseCursorClass();
                appCursor.SetCursor(2);

                int currentPage = DDP_Util.GetCurrentPage(); //Get current page number.
                DDP_Util.InitializePageCount(); //Initialize the page count.
                int pageCount = DDP_Util.GetPageCount(); //Get page count.
                //MessageBox.Show("Page " + currentPage + " of " + pageCount, "JambWow");
                treeViewPages.Nodes.Clear(); //Clear the treeview

                //Get an array of page names
                string[] ArrPageNames = IndexLayerUtil.GetPageNames();
                //Validate that the number of pages returned matches with Data Driven Pages.
                if (ArrPageNames == null) return;
                if (ArrPageNames.GetLength(0) == 0) return;
                if (ArrPageNames.GetLength(0) != pageCount)
                {
                    MessageBox.Show("The number of pages in your index layer does not match the number of pages in Data Driven Pages. Refresh DDP and try again.", "JambWow");
                    return;
                }

                //Get an array of page numbers
                string[] ArrPageNumbers = IndexLayerUtil.GetPageNumbers();
                if (ArrPageNumbers == null) return;
                if (ArrPageNumbers.GetLength(0) == 0) return;
                if (ArrPageNumbers.GetLength(0) != pageCount)
                {
                    MessageBox.Show("The number of pages in your index layer does not match the number of pages in Data Driven Pages. Refresh DDP and try again.", "JambWow");
                    return;
                }

                TreeNode rootNode = treeViewPages.Nodes.Add("Map Series", "Map Series");

                for (int n = 1; n <= pageCount; n++)
                {
                    string sPageName = ArrPageNames[n - 1];
                    string sPageNumber = ArrPageNumbers[n - 1];
                    string nodeKey = n.ToString();
                    string nodeText = nodeKey + " - " + sPageName + " (Page " + sPageNumber + ")";
                    TreeNode myNode = new TreeNode();
                    myNode.Name = nodeKey;
                    myNode.Text = nodeText;
                    //Uncheck the node if the page was marked as disabled.
                    myNode.Checked = PropertySetUtil.ContainsProperty(PageRangeUtil.GetDisabledPages(), sPageName);
                    //Add the new node.
                    rootNode.Nodes.Add(myNode);
                }
                rootNode.Expand();
                DisplayCurrentPageNumber();

                //Enable the navigation buttons
                toolStripFirstPage.Enabled = true;
                toolStripPrevPage.Enabled = true;
                toolStripNextPage.Enabled = true;
                toolStripLastPage.Enabled = true;

                //MessageBox.Show("Finished refreshing JambWow Navigation Window", "JambWow");
            }
            catch (Exception e)
            {
                GenUtil.ShowError(e);
            }
        }

        private void GoToSelectedPage(TreeNode myNode)
        {
            if (myNode.Parent == null) return;
            int pageNum = Convert.ToInt32(myNode.Name);
            //MessageBox.Show("Go to page: " + pageNum, "JambWow");
            DDP_Util.GoToPage(pageNum);
            DisplayCurrentPageNumber();
        }

        private void ShowContextMenu(TreeNodeMouseClickEventArgs e)
        {
            
            if (e.Button == MouseButtons.Right)
            {               
                TreeNode myNode = e.Node;
                //MessageBox.Show("Node name: " + myNode.Name, "JambWow");
                treeViewPages.SelectedNode = myNode;
                Point myPoint = new Point(e.X, e.Y);

                if (myNode.Parent == null) //If Map Series node
                    contextMenuMapSeries.Show(treeViewPages, myPoint);
                else //If page node
                    contextMenuPage.Show(treeViewPages, myPoint);
            }           
        }

        private void DisplayCurrentPageNumber()
        {
            //Set each node to the default colors, except for the node for the current page.
            //And get the current page node label
            string CurrentPageNodeLabel = "";
            TreeNode rootNode = treeViewPages.Nodes[0];

            foreach (TreeNode myNode in rootNode.Nodes)
            {
                if (myNode.Name != DDP_Util.CurrentKnownPage.ToString())
                {
                    myNode.BackColor = Color.Empty;
                    myNode.ForeColor = Color.Empty;
                    Font pNewFont = new Font(FontFamily.GenericSansSerif, (float) 8.25, FontStyle.Regular, GraphicsUnit.Point);
                    myNode.NodeFont = pNewFont;                    
                }
                else
                {
                    myNode.BackColor = Color.Wheat;
                    myNode.ForeColor = Color.DarkRed;
                    Font pNewFont = new Font(FontFamily.GenericSansSerif, (float)8.25, FontStyle.Bold, GraphicsUnit.Point);
                    myNode.NodeFont = pNewFont;
                    CurrentPageNodeLabel = myNode.Text;
                }
            }
            //Change the text that shows the current page
            lblCurrentPage.Text = "Current Page: " + CurrentPageNodeLabel;
        }

        private void UpdateEnabledDisabledPageStatus(int iPageIndex, bool bDisabled)
        {
            //Get the name of the page at the specified index
            string sPageName = IndexLayerUtil.GetPageName(iPageIndex);
            //Add or remove the page name from the list of disabled pages.
            if (bDisabled)
                PageRangeUtil.GetDisabledPages().SetProperty(sPageName, (object)sPageName);
            else
                PageRangeUtil.GetDisabledPages().RemoveProperty(sPageName);
        }

        #endregion       
    }
}
