﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;

using ESRI.ArcGIS.esriSystem;
using ESRI.ArcGIS.Framework;
using ESRI.ArcGIS.ArcMapUI;
using ESRI.ArcGIS.Carto;
using ESRI.ArcGIS.Geometry;
using ESRI.ArcGIS.ADF.Serialization;
using ESRI.ArcGIS.Catalog;
using ESRI.ArcGIS.Geodatabase;

namespace JAMBWOW
{
    public class JAMBWOW_Extension : ESRI.ArcGIS.Desktop.AddIns.Extension
    {        
        public static TempFileCollection TempFiles = new TempFileCollection();

        private static JAMBWOW_Extension s_extension;
        
        private const string PROPERTY_MESSAGE = "Message";
        private const string PROPERTY_DISABLED_PAGES = "Disabled Pages";
        private const string PROPERTY_FILE_NAME = "File Name";

        private IPropertySet2 m_ExtensionData;

        //Constructor
        public JAMBWOW_Extension()
        {
            s_extension = this;
        }

        //Call this function to display tool names in the JambWow toolbar (in message boxes)
        //Used for debugging purposes only.
        public static void ShowToolNamesInJambWowToolbar()
        {
            //Show the names and captions of items on the JambWow toolbar
            //Get a UID for the CommandBar
            UID JambWow_CommandBar_UID = new UID();
            JambWow_CommandBar_UID.Value = "USACE_JAMBWOW_USACE_JAMBWOW";
            //Get a colletion of command bars
            IDocument ThisDoc = ArcMap.Application.Document;
            ICommandBars CommandBars = ThisDoc.CommandBars as ICommandBars;
            //Get the specific command bar by UID
            ICommandBar JambWowCommandBar = CommandBars.Find(JambWow_CommandBar_UID, false, false) as ICommandBar;
            int count = JambWowCommandBar.Count;
            for (int i = 0; i < count; i++)
            {
                ICommandItem myItem = JambWowCommandBar[i];
                if (!(myItem is ICommandBar))
                {
                    ESRI.ArcGIS.SystemUI.ICommand myCommand = myItem.Command;
                    MessageBox.Show("ProgID: " + myItem.Name + Environment.NewLine + "Caption: " + myItem.Caption
                        + Environment.NewLine + "Category: " + myItem.Category
                        + Environment.NewLine + "Command: " + myCommand.Name, "JambWow");
                }
            }

        }

        //Use this function to get the JAMBWOW singleton, using lazy evaluation.
        internal static JAMBWOW_Extension GetExtension()
        {
            try
            {
                if (s_extension == null)
                {
                    UID extID = new UIDClass();
                    extID.Value = ThisAddIn.IDs.JAMBWOW_Extension;
                    ArcMap.Application.FindExtensionByCLSID(extID);
                }
                return s_extension;
            }
            catch (Exception pException)
            {
                GenUtil.ShowError(pException);
                return s_extension;
            }
        }

        #region protected Methods
        
        protected override void OnStartup()
        {
            try
            {
                //Event handlers
                ArcMap.Events.NewDocument += new IDocumentEvents_NewDocumentEventHandler(ArcMap_NewDocument);
                ArcMap.Events.OpenDocument += new IDocumentEvents_OpenDocumentEventHandler(ArcMap_OpenDocument);
                ArcMap.Events.BeforeCloseDocument += new IDocumentEvents_BeforeCloseDocumentEventHandler(ArcMap_BeforeCloseDocument);

                ////Add custom context menu buttons to the page layout element context menu(s) and TOC context menu.
                //try { CustomizeToolbarsAndContextMenus(); }
                //catch { }
            }
            catch (Exception pException)
            {
                GenUtil.ShowError(pException);
            }
        }

        protected override void OnSave(System.IO.Stream outStrm)
        {
            try
            {
                //********************************************************************************
                //Comment this add for now. Possibly use this functionality in a later version.
                //********************************************************************************
                ////If the document is not using relative paths, prompt the user.
                //if (ArcMap.Document.RelativePaths == false)
                //{
                //    DialogResult result = MessageBox.Show(
                //        "Would you like to set your map document to use relative paths?"
                //        + Environment.NewLine + "This is recommended for JambWow Map Series.",
                //        "Saving JambWow Map Series", MessageBoxButtons.YesNo);
                //    if (result == DialogResult.Yes)
                //        ArcMap.Document.RelativePaths = true; //Change document to use relative paths.
                //}

                ////***************************************
                ////Is there a benefit to setting the defaul geodatabase to my annotation database?
                ////
                ////Set the Default Geodatabase to be the geodatabase containing the annotation features.
                //IGxCatalogDefaultDatabase pDefaultDatabase = (IGxCatalogDefaultDatabase)ArcMap.Application;                
                //pDefaultDatabase.DefaultDatabaseName = AnnotationUtil.GetAnnoFeatWorkspaceName();
                ////***************************************
                
                //Remove the toolbox from ArcToolbox
                DDP_Util.RemoveToolbox();

                m_ExtensionData = new PropertySetClass();

                //********************************************************************************
                //Comment this add for now. Possibly use this functionality in a later version.
                //********************************************************************************
                ////Persist the map document file name.
                ////(This is what the file name WAS when the document was opened, what it will be after a "Save As")
                //IMapDocument pMapDoc = (IMapDocument)ArcMap.Document;
                //string sFileName = pMapDoc.DocumentFilename;
                ////MessageBox.Show("File Name: " + sFileName, "JambWow");
                //m_ExtensionData.SetProperty(PROPERTY_FILE_NAME, (object)sFileName);

                //Save a list of disabled pages.
                m_ExtensionData.SetProperty(PROPERTY_DISABLED_PAGES, PageRangeUtil.GetDisabledPages());

                ////Save a brief message with the map.
                //DialogResult result2 = MessageBox.Show("Would you like to save a brief message with the map document?",
                //    "Saving JambWow Map Series", MessageBoxButtons.YesNo);
                //if (result2 == DialogResult.Yes)
                //{
                //    //Prompt the user to save a text message
                //    string sMessage = "No Message";
                //    FormPromptUser pFormPromptUser = new FormPromptUser("Save a brief message:");
                //    pFormPromptUser.ShowDialog(GenUtil.GetAppWinRef());
                //    if (pFormPromptUser.DialogResult == DialogResult.OK)
                //        sMessage = pFormPromptUser.textInput.Text;
                //    m_ExtensionData.SetProperty(PROPERTY_MESSAGE, sMessage);
                //}

                PersistenceHelper.Save<IPropertySet2>(outStrm, m_ExtensionData);
            }
            catch (Exception pException)
            {
                GenUtil.ShowError(pException);
            }
        }

        protected override void OnLoad(System.IO.Stream inStrm)
        {
            try
            {
                PersistenceHelper.Load<IPropertySet2>(inStrm, ref m_ExtensionData);
            }
            catch (Exception pException)
            {
                GenUtil.ShowError(pException);
            }
        }

        #endregion

        #region private methods

        private void ArcMap_OpenDocument()
        {
            try
            {
                //Add custom context menu buttons to the page layout element context menu(s) and TOC context menu.
                CustomizeToolbarsAndContextMenus();

                //***********************************                
                //Load saved data from a propertyset.
                //***********************************

                //********************************************************************************
                //Comment this add for now. Possibly use this functionality in a later version.
                //********************************************************************************
                ////Get the persisted map document file name.
                //if (PropertySetUtil.ContainsProperty(m_ExtensionData, PROPERTY_FILE_NAME))
                //{
                //    string sOldFileName = (string)PropertySetUtil.GetPropertyValue<string>(m_ExtensionData, PROPERTY_FILE_NAME);
                //    //Get the new map document file name.
                //    IMapDocument pMapDoc = (IMapDocument)ArcMap.Document;
                //    string sNewFileName = pMapDoc.DocumentFilename;
                //    if (sOldFileName != sNewFileName)
                //        MessageBox.Show("The map document file name has changed."
                //            + Environment.NewLine + "The old file name was:"
                //            + Environment.NewLine + sOldFileName
                //            + Environment.NewLine + "The new file name is:"
                //            + Environment.NewLine + sNewFileName
                //            + Environment.NewLine
                //            + Environment.NewLine + "If your map contains annotation layers, please follow these steps:"
                //            + Environment.NewLine + "1. Rename the file geodatabase containing the annotation layers to match the naming convention of your map. This requires exiting out of ArcMap."
                //            + Environment.NewLine + "2. Repair the data sources for your annotation layers to point to the new file geodatabase name.",
                //            "JambWow");
                //}

                //Get a list of disabled pages, if any exist, or create a new list if not.
                PageRangeUtil.SetDisabledPage(PropertySetUtil.GetInnerPropSet(m_ExtensionData, PROPERTY_DISABLED_PAGES));

                ////Display a saved text message.
                //string sMessage = "";
                //if (PropertySetUtil.ContainsProperty(m_ExtensionData, PROPERTY_MESSAGE))
                //    sMessage = (string)PropertySetUtil.GetPropertyValue<string>(m_ExtensionData, PROPERTY_MESSAGE);
                //if (sMessage != "")
                //    MessageBox.Show("Saved message: " + sMessage, "JambWow - A brief message was saved with the map document.");

                //Update the Hide/Show button on the toolbar.
                ShowJAMBWOW_Button.ForceRefresh();
                
                //Clear and initialize the JambWow Dockable window.
                JAMBWOW_DockableWindow DW = JAMBWOW_DockableWindow.GetInstance();
                if (DW != null) 
                {
                    DW.ClearDialog(); 
                    //if (DDP_Util.DDPActivated()) DW.ActivateAndRefreshDialog();
                }
            }
            catch (Exception pException)
            {
                GenUtil.ShowError(pException);
            }
        }

        private void ArcMap_NewDocument()
        {
            try
            {
                //Add custom context menu buttons to the page layout element context menu(s) and TOC context menu.
                CustomizeToolbarsAndContextMenus();

                //Initialize a new list of disabled pages.
                PageRangeUtil.SetDisabledPage(new PropertySetClass());
                
                //Update the Hide/Show button on the toolbar.
                ShowJAMBWOW_Button.ForceRefresh();                
                
                //Clear and initialize the JambWow Dockable window.
                JAMBWOW_DockableWindow DW = JAMBWOW_DockableWindow.GetInstance();
                if (DW != null)
                {
                    DW.ClearDialog();
                    //if (DDP_Util.DDPActivated()) DW.ActivateAndRefreshDialog();
                }
            }
            catch (Exception pException)
            {
                GenUtil.ShowError(pException);
            }
        }

        private bool ArcMap_BeforeCloseDocument()
        {
            //Remove the toolbox from ArcToolbox
            DDP_Util.RemoveToolbox();
            
            //Clear the JambWow dockable window
            JAMBWOW_DockableWindow DW = JAMBWOW_DockableWindow.GetInstance();            
            if (DW != null) DW.ClearDialog();                

            //Close forms
            FormConvertLabelsToAnnotation.Close_Form();
            FormDeleteAnnotation.Close_Form();

            //Return True to abort the close process, or false to continue.
            return false; 
        }

        //Add custom context menu buttons to the page layout element context menu(s) and TOC context menu.
        private void CustomizeToolbarsAndContextMenus()
        {           
            ////Add Data Driven Pages Setup tool to the toolbar:            
            //AddItemToCommandBar("USACE_JAMBWOW_USACE_JAMBWOW", "esriArcMapUI.PageIndexSetupCommand", true);

            try
            {
                ////Add the Grid Index Generator to the toolbar:
                //AddItemToCommandBar("USACE_JAMBWOW_USACE_JAMBWOW", "[ Geoprocessing Tools ]_Grid Index Features", true);
                ////Add the Strip Map Index Generator to the toolbar:
                //AddItemToCommandBar("USACE_JAMBWOW_USACE_JAMBWOW", "[ Geoprocessing Tools ]_Strip Map Index Features", true);

                ////Add the Grid Index Generator to the toolbar:
                //AddItemToCommandBar("USACE_JAMBWOW_USACE_JAMBWOW", "Grid_Index_Features", true);
                ////Add the Strip Map Index Generator to the toolbar:
                //AddItemToCommandBar("USACE_JAMBWOW_USACE_JAMBWOW", "Strip_Map_Index_Features", true);
            }
            catch (Exception pException)
            {
                GenUtil.ShowError(pException);
            }

            ////Add the Dynamic Text Menu to the JambWow toolbar:
            //AddItemToCommandBar("USACE_JAMBWOW_USACE_JAMBWOW", "esriArcMapUI.DynamicTextMenu", true);
            
            ////Add the "Index Grid Generators" menu to the JambWow toolbar:
            //AddItemToCommandBar("USACE_JAMBWOW_USACE_JAMBWOW", "USACE_JAMBWOW_Index_Grid_Generators", true);

            //Add the "Show JambWow" Command to the Window menu
            AddItemToCommandBar("esriArcMapUI.MxWindowMenu", "USACE_JAMBWOW_ShowJAMBWOW_Button", false);
            //Change the icon of the "Show JambWow" command on the Window menu
            ShowJAMBWOW_Button.ChangeJambWowIconOnWindowMenu();

            //Add the "Convert Labels to Annotation" command to the TOC context menu.
            AddItemToCommandBar("esriArcMapUI.FeatureLayerContextMenu", "USACE_JAMBWOW_cmdLayerConvertToAnno", true);
            //Add the "Delete Annotation" command to the TOC context menu.
            AddItemToCommandBar("esriArcMapUI.FeatureLayerContextMenu", "USACE_JAMBWOW_cmdLayerDeleteAnnotation", false);
            //Add the "Convert Labels to Annoation for this page only" command to the TOC context menu.
            AddItemToCommandBar("esriArcMapUI.FeatureLayerContextMenu", "USACE_JAMBWOW_cmdLayerConvertToAnnoThisPageOnly", false);
            //Add the "Delete Annotation for  this page only" command to the TOC context menu.
            AddItemToCommandBar("esriArcMapUI.FeatureLayerContextMenu", "USACE_JAMBWOW_cmdLayerDeleteAnnotationThisPageOnly", false);



            //Add Tag/Untag Datagraph commands to LayoutContextMenu, ElementContextMenu, and MapFrameContextMenu
            AddItemToMultipleLayoutContexMenus("USACE_JAMBWOW_cmdTagDataGraph", true);
            AddItemToMultipleLayoutContexMenus("USACE_JAMBWOW_cmdUntagDataGraph", false);

            //Add "Show On This Page Only" command to LayoutContextMenu, ElementContextMenu, and MapFrameContextMenu
            AddItemToMultipleLayoutContexMenus("USACE_JAMBWOW_cmdShowOnThisPageOnly", true);
            //Add "Hide on select pages" command to LayoutContextMenu, ElementContextMenu, and MapFrameContextMenu
            AddItemToMultipleLayoutContexMenus("USACE_JAMBWOW_cmdHideOnSelectPages", false);
            //Add "Untag To Show On All Pages" command to LayoutContextMenu, ElementContextMenu, and MapFrameContextMenu
            AddItemToMultipleLayoutContexMenus("USACE_JAMBWOW_cmdShowOnAllPages", false);
            
            //Add "Position and size" tagging/untagging commands to LayoutContextMenu, ElementContextMenu, and MapFrameContextMenu
            AddItemToMultipleLayoutContexMenus("USACE_JAMBWOW_cmdTagStandardPositionAndSize", true);
            AddItemToMultipleLayoutContexMenus("USACE_JAMBWOW_cmdTagAlternatePositionAndSize", false);
            AddItemToMultipleLayoutContexMenus("USACE_JAMBWOW_cmdApplyStandardToThisPage", false);
            AddItemToMultipleLayoutContexMenus("USACE_JAMBWOW_cmdRemovePositonAndSizeTags", false);

            ////Hide the Data Driven Pages Toolbar
            //GenUtil.HideDataDrivenPagesToolbar();
        }

        //Add item to LayoutContextMenu, ElementContextMenu, and MapFrameContextMenu
        private void AddItemToMultipleLayoutContexMenus(string Item_ProgID, bool BeginNewGroup)
        {
            //Add item to Layout data frame element context menu. (Applies to data frame elements only.)
            AddItemToCommandBar("esriArcMapUI.MapFrameContextMenu", Item_ProgID, BeginNewGroup);
            //Add item to Layout context menu (no elements selected.
            AddItemToCommandBar("esriArcMapUI.LayoutContextMenu", Item_ProgID, BeginNewGroup);
            //Add item to Layout element context menu. (Does not apply to data frames).
            AddItemToCommandBar("esriArcMapUI.ElementContextMenu", Item_ProgID, BeginNewGroup);
        }

        //Add an item (command, tool, or sub-menu) to a CommandBar (toolbar or context menu).
        private void AddItemToCommandBar(string CommandBar_ProgID, string Item_ProgID, bool BeginNewGroup)
        {
            //Get a UID for the item
            UID Item_UID = new UID();
            Item_UID.Value = Item_ProgID;
            //Get a UID for the CommandBar
            UID CommandBar_UID = new UID();
            CommandBar_UID.Value = CommandBar_ProgID;
            //Get a colletion of command bars
            IDocument ThisDoc = ArcMap.Application.Document;
            ICommandBars CommandBars = ThisDoc.CommandBars as ICommandBars;
            //Get the specific command bar by UID
            ICommandBar MyCommandBar = CommandBars.Find(CommandBar_UID, false, false) as ICommandBar;
            //Determine if the command bar already contains the item, and if not, add it.
            ICommandItem Item = MyCommandBar.Find(Item_UID, false);
            if (!(Item != null))
            {
                object Missing = Type.Missing;
                MyCommandBar.Add(Item_UID, ref Missing);
                Item = MyCommandBar.Find(Item_UID, false);
                Item.Group = BeginNewGroup; //Set the group property to true to indicate that this is the beginning of a group.
            }
        }

        #endregion
    }
}
