﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using ESRI.ArcGIS.esriSystem;

namespace JAMBWOW
{
    public static class PageRangeUtil
    {        
        public enum PageRangeTypes { All, PageList };

        private static IPropertySet2 m_DisabledPages;

        public static IPropertySet2 GetDisabledPages()
        {
            if (m_DisabledPages == null) m_DisabledPages = new PropertySetClass();
            return m_DisabledPages;
        }

        public static void SetDisabledPage(IPropertySet2 DisabledPages)
        {
            m_DisabledPages = DisabledPages;
        }

        public static List<int> ParsePageNumbers(string sPageNumList)
        {
            bool FoundInvalidText = false;
            List<int> pageNumbers = new List<int>();
            //Split the list into comma delimited tokens
            string[] arrTokens = sPageNumList.Split(new Char[] { ',' });
            foreach (string sToken in arrTokens)
            {
                string sTrimToken = sToken.Trim();
                int iPageNum;
                bool isNum = int.TryParse(sTrimToken, out iPageNum);
                if (isNum) //If the token is numeric, it is a page number
                {
                    if (!pageNumbers.Contains(iPageNum)) pageNumbers.Add(iPageNum);
                }
                else //If the token is not numeric, it might be a page range.
                {
                    //Split this token into two numeric tokens seperated by a dash.
                    string[] arrRangeTokens = sTrimToken.Split(new Char[] { '-' });
                    //If there are exactly two tokens, and they are both numeric,
                    //Then this is a page range.
                    if (arrRangeTokens.Length == 2)
                    {
                        string sTrimRangeToken1 = arrRangeTokens[0].Trim();
                        string sTrimRangeToken2 = arrRangeTokens[1].Trim();
                        int iRange1;
                        int iRange2;
                        bool isRangeNum1 = int.TryParse(sTrimRangeToken1, out iRange1);
                        bool isRangeNum2 = int.TryParse(sTrimRangeToken2, out iRange2);
                        if (isRangeNum1 && isRangeNum2)
                        {
                            for (int x = iRange1; x <= iRange2; x++)
                            {
                                pageNumbers.Add(x);
                            }
                        }
                        else FoundInvalidText = true;
                    }
                    else FoundInvalidText = true;
                }
            }
            if (FoundInvalidText) MessageBox.Show("Encountered problems parsing the page number text. Some or all of the pages may not be processed.", "JambWow");
            return pageNumbers;
        }

        public static List<int> RemoveDisabledPages(List<int> PageNumbers)
        {
            List<int> NewPageNumbersList = new List<int>();
            foreach (int iPageNumber in PageNumbers)
            {
                //Get the page name
                string sPageName = IndexLayerUtil.GetPageName(iPageNumber);
                //Determine if the page is disabled.
                bool bEnabled = !PropertySetUtil.ContainsProperty(m_DisabledPages, sPageName);
                //If the page is enabled, add it to the list
                if (bEnabled) NewPageNumbersList.Add(iPageNumber);               
            }
            return NewPageNumbersList;
        }
    }
}
