﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using ESRI.ArcGIS.esriSystem;

namespace JAMBWOW
{
    public partial class ProgressWindow : Form
    {        
        public ITrackCancel CancelTracker;

        private bool m_bCancel;
        private int m_iPercentCompletePrev = 0;
        private string m_ActionText;

        //Constructor
        public ProgressWindow(string ActionText)
        {
            InitializeComponent();
            m_ActionText = ActionText; //Text to show on the dialog, such as "Exporting Page..."
        }

        //Destructor
        ~ProgressWindow()
        {
        }

        private void cmdCancel_Click(object sender, EventArgs e)
        {
            m_bCancel = true;
            this.lblNumPointsProcessed.Text = "Canceling...";
            this.lblPercentComplete.Text = "";
            
            //If an action is being tracked by the cancel tracker, cancel the action.
            if (CancelTracker != null) CancelTracker.Cancel(); 

            //Allow the UI to refresh
            System.Windows.Forms.Application.DoEvents();
        }

        //This handles the Progress Changed Event.
        //Return true if the operation was canceled.
        public bool ProgressChanged(ProgressReport pProgressReport, bool bSimple = false)
        {
            if (bSimple)
            {
                this.lblNumPointsProcessed.Text = m_ActionText;                
                this.lblPercentComplete.Visible = false;
                this.progressBar1.Visible = false;
                this.cmdCancel.Visible = false;
                return false;
            }
            
            bool Canceled = false;//Initialize Cancel to false
            int iPercentComplete = 0;
            try
            {
                //Cacluate % complete.
                int iCurrentItemNumber = pProgressReport.CurrentItemNumber;
                int iNumberOfItems = pProgressReport.NumberOfItems;
                double dPercentComplete = ((double)(iCurrentItemNumber-1) * (double)100) / iNumberOfItems;
                iPercentComplete = (int) Math.Floor(dPercentComplete);
                
                //Update the progress bar and text
                this.progressBar1.Value = iPercentComplete;
                this.lblNumPointsProcessed.Text = m_ActionText + " " + iCurrentItemNumber + " of " + iNumberOfItems;
                this.lblPercentComplete.Text = iPercentComplete + "% Complete";
            }
            catch (Exception pException)
            {
                //Catch any unexpected errors that bubble up to this scope level.
                string errMessage = "An unexpected error occured while trying to display a progress update: \n\n"
                    + pException;
                //Show the full contents of the exception, inner exception, and stack trace.
                MessageBox.Show(errMessage, "JambWow - Unexpected Error, while trying to display a progress update");
            }

            //Allow the UI to refresh (if the progress went up by a percent or more).            
            if (iPercentComplete != m_iPercentCompletePrev)
                System.Windows.Forms.Application.DoEvents();

            //Store previous percent complete
            m_iPercentCompletePrev = iPercentComplete;

            //If the cancel button was clicked, abort the load.
            if (m_bCancel == true) Canceled = true;
            return Canceled;
        }
    }
}
