import arcpy
from arcpy import env
import math
import cmath
import os
from math import floor
import sys
import Tkinter
import tkMessageBox
import arcgisscripting


def LayerSelectionCount(_in_polyline_m_layer):

    """
    ----------------------------------------------------------------------------
    Gets the number of selected features from the input feature layer. Only one
    feature may be selected upon running the toolbox.
    ----------------------------------------------------------------------------
    """

    count = int(arcpy.GetCount_management(_in_polyline_m_layer).getOutput(0))
    if (count == 1):
        arcpy.AddMessage("Confirmed; one selection in layer. You may proceed.")
    else:
        root = Tkinter.Tk()
        root.withdraw()
        tkMessageBox.showwarning("Error", "The number of selected features is: "
                                 + str(count) + ". Should only have one selected feature.")
        quit()
    

def GetLayerType(desc):

    """
    ----------------------------------------------------------------------------
    Retrieves the layer type of the input feature layer. Verifies that the
    layer type is a polyline, as other layer types are not supported.
    ----------------------------------------------------------------------------
    """

    LayerType = desc.shapeType
    if (LayerType == "Polyline"):
        arcpy.AddMessage("Confirmed; layer is a polyline. You may proceed.")
    else:
        root = Tkinter.Tk()
        root.withdraw()
        tkMessageBox.showwarning("Not an efficient layer type.")
        quit()

def ValidInputGeometry():

    """
    ----------------------------------------------------------------------------
    Checks the input feature layer to verify that it is M-Aware, or in other
    words, checks to see if it has associated M values.
    ----------------------------------------------------------------------------
    """

    is_valid_poly_line = False
    is_m_enabled = False
    msg = None

    rows = arcpy.SearchCursor(_in_polyline_m_layer)
    for row in rows:
        feat = row.getValue(shapefieldname)
        if feat.type == "polyline":
            if desc.hasM:
                is_valid_poly_line = True
                msg = "Polyline is M-Aware. You may continue."
            else:
                msg = "The input geometry is not M-Aware, cannot generate your shapefile."
                quit()
        else:
            msg = "The input geometry is not a polyline. Cannot generate your shapefile."
            root = Tkinter.Tk()
            root.withdraw()
            tkMessageBox.showwarning("Invalid Geometry", "The input geometry is not a polyline. Cannot generate your shapefile.")
            quit()

    return (is_valid_poly_line, msg)


def MultiPartCheck():

    """
    ----------------------------------------------------------------------------
    Checks the input feature layer to verify that it is not a multi part
    feature; as multipart features are not supported in this version.
    ----------------------------------------------------------------------------
    """

    PartCount = 0
    rows = arcpy.SearchCursor(_in_polyline_m_layer)
    for row in rows:
        feat = row.getValue(shapefieldname)
        for part in feat:
            PartCount = PartCount + 1

        if PartCount > 1:
            arcpy.AddMessage("This is a multi part shapefile -- Not supported.")
            arcpy.AddMessage("Please choose another shapefile.")
            root = Tkinter.Tk()
            root.withdraw()
            tkMessageBox.showwarning("Multi Part", "This is a multi part shapefile -- Not supported.")

            quit()
        else:
            print "Selefted file is ready. You may proceed!"



def PolyWidthValidation(PolygonWidth, _StationInterval):

    """
    ----------------------------------------------------------------------------
    Validates the polygon width. As a general guideline, the polygon width
    should be 12-15% larger than the length of the stationing siplayed, with
    10% being the absolute minimum. In the event of an error, user is given
    option to ignore warning and continue generating polygon or stop program
    to adjust input parameters.
    ----------------------------------------------------------------------------
    """

    MinWidth = _StationInterval + (_StationInterval * 0.10)
    MaxWidth = _StationInterval + (_StationInterval * 0.15)

    if (PolygonWidth < MinWidth):
        root = Tkinter.Tk()
        root.withdraw()
        answer = tkMessageBox.askquestion("Width Validation Error", "Warning: Polygon width needs to be at least 10% larger than the station " +
        "interval. Your current parameters may result in polygons that don not capture the entire station interval. Do you wish to continue?" +
        "\n\n Data Frame Widht: " + str(_DataFrameWidth) + "\n Station Interval: " + str(_StationInterval) + "\n Scale: " + str(_Scale))
        if (answer == 'yes'):
            root = Tkinter.Tk()
            root.withdraw()
            tkMessageBox.showwarning("Width Validation Error", "Continuing with generating your polygon.")
        elif (answer == 'no'):
            root = Tkinter.Tk()
            root.withdraw()
            tkMessageBox.showwarning("Width Validation Error", "Please adjust your data frame, scale, or station interval.")
            quit()
    elif (PolygonWidth > MaxWidth):
        root = Tkinter.Tk()
        root.withdraw()
        answer = tkMessageBox.askquestion("Width Validation Error", "Warning: Polygon width exceeds the standard length (15%) of the station " +
        "interval. Your current parameters may result in polygons that exceed the normal page. Do you wish to continue?" +
        "\n\n Data Frame Widht: " + str(_DataFrameWidth) + "\n Station Interval: " + str(_StationInterval) + "\n Scale: " + str(_Scale))
        if (answer == 'yes'):
            root = Tkinter.Tk()
            root.withdraw()
            tkMessageBox.showwarning("Width Validation Error", "Continuing with generating your polygon.")
        elif (answer == 'no'):
            root = Tkinter.Tk()
            root.withdraw()
            tkMessageBox.showwarning("Width Validation Error", "Please adjust your data frame, scale, or station interval.")
            quit()
    else:
        arcpy.AddMessage("Polygon width falls within standards of station interval.")
       

def InterpolateByStation(feat, _StationInterval):

    """
    ----------------------------------------------------------------------------
    Starting at the 'From Station' M value, the function traverses the polyline
    and returns the point location at the desired M value. If an exact match
    is not found, retrieve the point before and after the desired M value and
    interpolate to find a point location for the desired M value.
    ----------------------------------------------------------------------------
    """

    first_m = None
    first_y = None
    first_x = None
    first_z = None
    interpolated_x = None
    interpolated_y = None
    interpolated_m = None
    count = 0
    i = 0

    last_m = None
    last_x = None
    last_y = None
    last_z = None
    match = 0
    exact_match = 0

    for pnt in feat.getPart(0):
        if (abs(pnt.M - _StationInterval)) < 0.0001:
            interpolated_x = round(pnt.X,3)
            interpolated_y = round(pnt.Y,3)
            interpolated_m = pnt.M
            match = 1
            exact_match = 1
        elif (pnt.M < _StationInterval):
            first_m = round(pnt.M,3)
            first_x = round(pnt.X,3)
            first_y = round(pnt.Y,3)
        else:
            if(match == 0):
                last_x = round(pnt.X,3)
                last_y = round(pnt.Y,3)
                last_m = pnt.M
                match = 1
                break
        if match == 1:
            break
    if exact_match == 0:

        weighted_distance = (_StationInterval - first_m)/(last_m - first_m)
        interpolated_x = round(first_x + (weighted_distance * (last_x - first_x)),3)
        interpolated_y = round(first_y + (weighted_distance * (last_y - first_y)),3)
        interpolated_m = _StationInterval

    return (interpolated_x, interpolated_y, interpolated_m)


def GetSegmentPoints(feat, StartingStation, _StationInterval):

    """
    ----------------------------------------------------------------------------
    Function retrieves and stores each point along in the polyline between your
    "From" and "To" station in an array. The array is returned and used in
    determining the 'INGRID' attribute; i.e. if the entire polyline segment fits
    withing the grid of the polygon.
    ----------------------------------------------------------------------------
    """
    _ar = arcpy.Array()
    count = 0
    for pnt in feat.getPart(0):
        if pnt and pnt.M > StartingStation and pnt.M <= _StationInterval:
            count = count + 1
            _ar.add(arcpy.Point(pnt.X,pnt.Y,pnt.M))
            
    return(count, _ar)


def Dot(o, p, pp):

    """
    ----------------------------------------------------------------------------
    Intermediate function in helping calculate a projected point for the left
    and right offset of a specific segment of the polyline.
    ----------------------------------------------------------------------------
    """
    
    dotp = (p[0] - o[0]) * (pp[0] - o[0]) + (p[1]-o[1]) * (pp[1] - o[1])
    m2 = math.sqrt((pp[0] - o[0]) * (pp[0] - o[0]) + (pp[1] - o[1]) * (pp[1] - o[1]))
    ndotp = dotp/(m2*m2)

    return ndotp


def CalculateProjectedPoint(firstXinSegment, firstYinSegment, IPx, IPy, pntX, pntY):

    """
    ----------------------------------------------------------------------------
    Looking at a particular segment of the polyline, we calculate the orthogonal
    distance from a point either above or below the segment to a point on the
    segment.
    ----------------------------------------------------------------------------
    """

    distance = 0
    dp = Dot((firstXinSegment, firstYinSegment), (pntX, pntY), (IPx, IPy))
    if dp >=0 and dp <= 1:
        x = firstXinSegment + dp * (IPx - firstXinSegment)
        y = firstYinSegment + dp * (IPy - firstYinSegment)
        distance = math.sqrt(math.pow((pntX - x), 2) + math.pow((pntY - y), 2))
    else:
        print "error: cannot project"

    return distance



def CalculateOffset(feat, firstXinSegment, firstYinSegment, IPx, IPy, StartingStation, _StationInterval, PolygonHeight):

    """
    ----------------------------------------------------------------------------
    Function calculates the offset, or maximum distance from a point, either
    above or below, to the specific polygon segment. This allows for the page
    to be centered by the "polyline line segment" and not cenetered by the
    "straight line segment".
    ----------------------------------------------------------------------------
    """

    max_left_distance = 0
    left_count = 0
    max_right_distance = 0
    right_count = 0

    for pnt in feat.getPart(0):
        if pnt and pnt.M > StartingStation and pnt.M <= _StationInterval:
            pntX = pnt.X
            pntY = pnt.Y
            # Determine which side the point is on
            V1x = IPx - firstXinSegment
            V2x = IPx - pntX
            V1y = IPy - firstYinSegment
            V2y = IPy - pntY
            xp = (V1x * V2y) - (V1y * V2x)
            if ((IPx - firstXinSegment) < 0):
                #print "Moving from right to left"
                if (xp > 0):
                    #print "Point is above the line(left)"
                    left_count = left_count + 1
                    GetDistance = CalculateProjectedPoint(firstXinSegment, firstYinSegment, IPx, IPy, pntX, pntY)
                    if (GetDistance > max_left_distance):
                        max_left_distance = GetDistance
                elif (xp < 0):
                    #print "Point is below the line(right)"
                    right_count = right_count + 1
                    GetDistance = CalculateProjectedPoint(firstXinSegment, firstYinSegment, IPx, IPy, pntX, pntY)
                    if (GetDistance > max_right_distance):
                        max_right_distance = GetDistance
            elif ((IPx - firstXinSegment) > 0):
                #print "Moving from left to right"
                if (xp > 0):
                    #print "Point is below the line(right)"
                    right_count = right_count + 1
                    GetDistance = CalculateProjectedPoint(firstXinSegment, firstYinSegment, IPx, IPy, pntX, pntY)
                    if (GetDistance > max_right_distance):
                        max_right_distance = GetDistance
                elif (xp < 0):
                    #print "Point is above the line(left)"
                    left_count = left_count + 1
                    GetDistance = CalculateProjectedPoint(firstXinSegment, firstYinSegment, IPx, IPy, pntX, pntY)
                    if (GetDistance > max_left_distance):
                        max_left_distance = GetDistance

    MaxLeftOffset = (max_left_distance/2)
    MaxRightOffset = (max_right_distance/2)
    
    HorizontalOffset = MaxLeftOffset + (MaxRightOffset * -1)
    HorizOffsetLimit = (PolygonHeight/2) * 0.90
    if (HorizontalOffset > HorizOffsetLimit):
        HorizontalOffset = HorizOffsetLimit
    elif (HorizontalOffset < (HorizOffsetLimit * -1)):
        HorizontalOffset = (HorizOffsetLimit * -1)
        
    return (HorizontalOffset)



def GetSegmentSlope(firstXinSegment, firstYinSegment, IPx, IPy):

    """
    ----------------------------------------------------------------------------
    Calculates the slope of the segment, the angle in radians and the
    rotation angle. Function takes in the segments 'From' and 'To' vertices.
    ----------------------------------------------------------------------------
    """
    

    M = (IPy - firstYinSegment) / (IPx - firstXinSegment)
    RadAngle = math.atan2((IPy - firstYinSegment),(IPx - firstXinSegment))

    if (IPx - firstXinSegment) < 0:
        RotationAngle = round(math.atan2((IPy - firstYinSegment),(IPx - firstXinSegment)) * (180/(math.pi* -1)), 4)
    elif (IPx - firstXinSegment) > 0:
        RotationAngle = 180 - round(math.atan2((IPx - firstXinSegment),(IPy - firstYinSegment)) * (180/(math.pi* -1)), 4) + 90
    
    return (M, RotationAngle, RadAngle)



def AddSegmentBuffer(firstXinSegment, firstYinSegment, IPx, IPy, PolygonWidth, RadAngle):
    
    """
    ----------------------------------------------------------------------------
    Based on the polygon width and the distance of the segment, a buffer is
    added to the width of each polygon. The buffer allows for additional area on
    each side of the map page to be seen.
    ----------------------------------------------------------------------------
    """
  
    distance = math.sqrt(math.pow((IPx - firstXinSegment),2) + math.pow((IPy - firstYinSegment),2))
    BuffWid = ((PolygonWidth - distance)/2)
    
    if (IPx - firstXinSegment) <> 0:
        c_x = firstXinSegment - (math.cos(RadAngle) * BuffWid)
        c_y = firstYinSegment - (math.sin(RadAngle) * BuffWid)
        
        d_x = IPx + (math.cos(RadAngle) * BuffWid)
        d_y = IPy + (math.sin(RadAngle) * BuffWid)

    elif (IPx - firstXinSegment) == 0:
        c_x = firstXinSegment
        d_x = IPx
        if (IPy > firstYinSegment):
            d_y = IPy + BuffWid
            c_y = firstYinSegment - BuffWid     
        else:
            d_y = IPy - BuffWid             
            c_y = firstYinSegment + BuffWid
    
    return (c_x, c_y, d_x, d_y)


    
def CalculateGeometries(FromCoordX, FromCoordY, ToCoordX, ToCoordY, Slope, UpperHeight, LowerHeight):

    """
    ----------------------------------------------------------------------------
    Function calculates the geometries (x and y coords) for each vertex of the
    polygon.
    ----------------------------------------------------------------------------
    """
    
    
    angle = math.atan(Slope)
    p1x = FromCoordX + (math.sin(angle) * LowerHeight)
    p2x = FromCoordX - (math.sin(angle) * UpperHeight)
    p3x = ToCoordX - (math.sin(angle) * UpperHeight)
    p4x = ToCoordX + (math.sin(angle) * LowerHeight)

    p1y = FromCoordY - (math.cos(angle) * LowerHeight)
    p2y = FromCoordY + (math.cos(angle) * UpperHeight)
    p3y = ToCoordY + (math.cos(angle) * UpperHeight)
    p4y = ToCoordY - (math.cos(angle) * LowerHeight)

        
    return (p1x, p2x, p3x, p4x, p1y, p2y, p3y, p4y)


def AddGeometries(a,b,c,d,e,f,g,h):

    """
    ----------------------------------------------------------------------------
    Function takes the four vertices of the polygon and stores them in array
    so they can be added to the shapefile.
    ----------------------------------------------------------------------------
    """
    
    coords = [(a,e), (b,f), (c,g), (d,h), (a,e)]
    ar = arcpy.Array()
    
    for x,y in coords:
        ar.add(arcpy.Point(x,y))
        
    return ar



def LastPageExtend(_StationInterval, lastMinPoly, firstYinSegment, firstXinSegment, lastXinPoly, lastYinPoly):

    """
    ----------------------------------------------------------------------------
    Function to extend the profile view of the map, as the last page of the map
    series, in most cases, will be derived from the latter or left over piece
    of the entire polyline. The profile view will center on the page length of
    the page and not on the remainder of the polyline.
    ----------------------------------------------------------------------------
    """
    
    lpangle = math.atan2((lastYinPoly - firstYinSegment),(lastXinPoly - firstXinSegment))
    DeltaM = _StationInterval - lastMinPoly
    lpX = lastXinPoly + DeltaM *(math.cos(lpangle)) 
    lpY = lastYinPoly + DeltaM * (math.sin(lpangle))

    return (lpX, lpY)

"""
def FormatBeginStation(StartingStation, precision=None):

    
    ----------------------------------------------------------------------------
    Function formats the station values appending the correct amount of 0's
    and correct number of decimal places.
    ----------------------------------------------------------------------------

    
    begin_station = str(int(StartingStation))
    precision = 0
    if len(begin_station) == (1+precision):
        print "case1"
        levee_station_code = "0+0%s" % begin_station[0:(1+precision)]
    elif len(begin_station) == (2+precision):
        print "case2"        
        levee_station_code = "0+%s" % begin_station[0:(2+precision)]
    else:
        print "case3"
        print len(begin_station)
        print (2+precision)
        print begin_station[0:len(begin_station)-(2+precision)]
        levee_station_code = "%s+%s" % (begin_station[0:len(begin_station)-(2+precision)], begin_station[len(begin_station)-(2+precision):len(begin_station)])
        print levee_station_code
    return (levee_station_code)
    """

def FormatBeginStation (esri__measure, precision=None):  

    """
    ----------------------------------------------------------------------------
    Alternate function to format the station values, appending the correct
    amount of 0's and correct number of decimal places.
    ----------------------------------------------------------------------------
    """
    FindLabel = None

    if esri__measure >= 0:
        if (round (esri__measure - int (esri__measure / 100) * 100, 0) > 0):
          if (round (esri__measure - int (esri__measure / 100) * 100, 0) < 10):
            FindLabel = str(int (esri__measure / 100)) + "+0" + str(int (round (esri__measure - int (esri__measure / 100) * 100, 0)))
          else:
            FindLabel = str(int (esri__measure / 100)) + "+" + str(int (round (esri__measure - int (esri__measure / 100) * 100, 0)))
        else:
          FindLabel = str(int (esri__measure / 100)) + "+00"
    else:
        if (round (abs(esri__measure) - int (abs(esri__measure)  / 100) * 100, 0) > 0):
          if (round (abs(esri__measure)  - int (abs(esri__measure)  / 100) * 100, 0) < 10):
            FindLabel = "-" + str(int (abs(esri__measure)  / 100)) + "+0" + str(round (abs(esri__measure) - int (abs(esri__measure) / 100) * 100, 0))
          else:
            FindLabel = "-" + str(int (abs(esri__measure) / 100)) + "+"  + str(round (abs(esri__measure) - int (abs(esri__measure) / 100) * 100, 0))
        else:
          FindLabel = "-" + str(int (abs(esri__measure) / 100)) + "+00"

    return (FindLabel)


# Main

firstXinSegment = None
firstYinSegment = None
StartingStation = None
_UnitsRatio = 0.083333333333333333333
PageCount = 1

# Toolbox Parameters
_in_polyline_m_layer = arcpy.GetParameterAsText(0)
_StationInterval = float(arcpy.GetParameterAsText(1))
_IncreaseBy = _StationInterval
_Scale = float(arcpy.GetParameterAsText(2))
_DataFrameHeight = float(arcpy.GetParameterAsText(3))
_DataFrameWidth = float (arcpy.GetParameterAsText(4))
_MapName = arcpy.GetParameterAsText(5)
_ResultFile = arcpy.GetParameterAsText(6)
arcpy.env.workspace = "C:\Users\b2edsjlp\Desktop\Algorithm"

PolygonHeight = _DataFrameHeight * _Scale * _UnitsRatio
PolygonWidth = _DataFrameWidth * _Scale * _UnitsRatio


desc = arcpy.Describe(_in_polyline_m_layer)
shapefieldname = desc.ShapeFieldName
spatial_reference = arcpy.Describe(_in_polyline_m_layer).spatialReference
rows = arcpy.SearchCursor(_in_polyline_m_layer)

# Validation Functions
LayerSelectionCount(_in_polyline_m_layer)
GetLayerType(desc)
(valid, msg) = ValidInputGeometry()
print str(valid) + ".", msg
MultiPartCheck()
PolyWidthValidation(PolygonWidth, _StationInterval)


print "Your shapefile is being generated."

out_path = os.path.split(_ResultFile)[0]
out_name = os.path.split(_ResultFile)[1]

if (os.path.exists(_ResultFile)):
    root = Tkinter.Tk()
    root.withdraw()
    result = tkMessageBox.askquestion("Overwrite File?", "This file already exists. Do you wish to overwrite it?")
    if result =='yes':
        arcpy.DeleteFeatures_management(_ResultFile)
    else:
        root = Tkinter.Tk()
        root.withdraw()
        tkMessageBox.showwarning("Error", "Please select a different file name/path.")
else:
    arcpy.CreateFeatureclass_management(out_path, out_name, 'POLYGON', "","","",spatial_reference)

# Table Attributes
arcpy.AddField_management(os.path.join(out_path, out_name), "SMAP_NAME", "TEXT", 255)
arcpy.AddField_management(os.path.join(out_path, out_name), "SMAP_ANGLE", "DOUBLE", 7, 4)
arcpy.AddField_management(os.path.join(out_path, out_name), "SMAP_NUM", "LONG", 9)
arcpy.AddField_management(os.path.join(out_path, out_name), "SMAP_SCALE", "LONG", 9)
arcpy.AddField_management(os.path.join(out_path, out_name), "FROM_STAT", "DOUBLE", 15, 3)
arcpy.AddField_management(os.path.join(out_path, out_name), "TO_STAT", "DOUBLE", 15, 3)
arcpy.AddField_management(os.path.join(out_path, out_name), "IN_GRID", "TEXT", 1)
arcpy.AddField_management(os.path.join(out_path, out_name), "FROM_TO_ST", "TEXT", 255)

cur = arcpy.InsertCursor(os.path.join(out_path, out_name))
firstPageFlag = True
for row in rows:
    feat = row.getValue(shapefieldname)
    pnt = feat.getPart(0)

    # Initialize starting (x,y,m) points
    firstXinSegment = feat.firstPoint.X
    firstYinSegment = feat.firstPoint.Y
    firstMinPoly = feat.firstPoint.M
    lastXinPoly = feat.lastPoint.X
    lastYinPoly = feat.lastPoint.Y
    lastMinPoly = feat.lastPoint.M
    
    StartingStation = firstMinPoly
    isLastPage = False 
    while isLastPage == False:
        if (_StationInterval < lastMinPoly):   
            print "==================================BEGIN NEW ITERATION"
            (IPx, IPy, IPm) = InterpolateByStation(feat, _StationInterval)
            (HorizontalOffset) = CalculateOffset(feat, firstXinSegment, firstYinSegment, IPx, IPy, StartingStation, _StationInterval, PolygonHeight)
            UpperHeight = (PolygonHeight/2) + HorizontalOffset
            LowerHeight = (PolygonHeight/2) - HorizontalOffset
            (count, _ar) = GetSegmentPoints(feat, StartingStation, _StationInterval)
            (Slope, RotationAngle, RadAngle) = GetSegmentSlope(firstXinSegment, firstYinSegment, IPx, IPy)
            (FromCoordX, FromCoordY, ToCoordX, ToCoordY ) = AddSegmentBuffer(firstXinSegment, firstYinSegment, IPx, IPy, PolygonWidth, RadAngle)
            (a,b,c,d,e,f,g,h) = CalculateGeometries(FromCoordX, FromCoordY, ToCoordX, ToCoordY, Slope, UpperHeight,LowerHeight)
            arrayOfPoints = AddGeometries(a,b,c,d,e,f,g,h)
            RA = (RotationAngle)
        else:
            print "==================================BEGIN FINAL ITERATION"
            (HorizontalOffset) = CalculateOffset(feat, firstXinSegment, firstYinSegment, lastXinPoly, lastYinPoly, StartingStation, _StationInterval, PolygonHeight)
            UpperHeight = (PolygonHeight/2) + HorizontalOffset
            LowerHeight = (PolygonHeight/2) - HorizontalOffset
            (count, _ar) = GetSegmentPoints(feat, StartingStation, _StationInterval)
            (FinalSlope, RotationAngle, RadAngle) = GetSegmentSlope(firstXinSegment, firstYinSegment, lastXinPoly, lastYinPoly)
            (lpX, lpY) = LastPageExtend(_StationInterval, lastMinPoly, firstYinSegment, firstXinSegment, lastXinPoly, lastYinPoly)
            (x0, y0, x1, y1) = AddSegmentBuffer(firstXinSegment, firstYinSegment, lpX, lpY, PolygonWidth, RadAngle)
            (a,b,c,d,e,f,g,h) = CalculateGeometries(x0, y0, x1, y1, FinalSlope, UpperHeight, LowerHeight)
            arrayOfPoints = AddGeometries(a,b,c,d,e,f,g,h)
            RA = (RotationAngle)
            isLastPage = True

        NewAttr = cur.newRow()
        NewAttr.shape = arrayOfPoints
        polylineSegmentTemp = arcpy.Polyline(_ar)
        polygonTemp = arcpy.Polygon(arrayOfPoints)
        b_Contains = polygonTemp.contains(polylineSegmentTemp)
        print b_Contains
        
        NewAttr.SMAP_NAME = str(_MapName) +"_{0:03d}".format(PageCount)
        NewAttr.SMAP_ANGLE = RA
        NewAttr.SMAP_NUM = PageCount
        NewAttr.SMAP_SCALE = _Scale
        NewAttr.FROM_STAT = StartingStation
        NewAttr.TO_STAT = _StationInterval 

        if b_Contains == True:
            NewAttr.IN_GRID = 'Y'
        else:
            NewAttr.IN_GRID = 'N'

        if isLastPage == True:
            EndingStation = lastMinPoly
        else:
            EndingStation = _StationInterval

        lev_sta_code_end = FormatBeginStation(EndingStation)
        if firstPageFlag == True:
            lev_sta_code = FormatBeginStation(StartingStation, 3)
            firstPageFlag = False

        else:
            lev_sta_code = FormatBeginStation(StartingStation)

        NewAttr.FROM_TO_ST ="STA. " + lev_sta_code + " to STA. " + lev_sta_code_end
        
        cur.insertRow(NewAttr)
        arrayOfPoints.removeAll()
        
        
        StartingStation = _StationInterval
        _StationInterval = _StationInterval + _IncreaseBy
    
        firstXinSegment = IPx
        firstYinSegment = IPy
        PageCount = PageCount + 1
        
del cur
