﻿using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Windows.Forms;
using System.Drawing;
using ESRI.ArcGIS.Framework;
using ESRI.ArcGIS.esriSystem;

namespace JAMBWOW
{
    public class ShowJAMBWOW_Button : ESRI.ArcGIS.Desktop.AddIns.Button
    {
        private static ShowJAMBWOW_Button m_singleton = null;
        private static bool m_Dockable_Window_Visible = false;
        private static ICommandItem m_HideShowButton;
        private static Bitmap m_OpenBookImage = new Bitmap(GenUtil.GetProgramPath() + @"\Images\ShowJAMBWOW_Button_Open.png");
        private static Bitmap m_ClosedBookImage = new Bitmap(GenUtil.GetProgramPath() + @"\Images\ShowJAMBWOW_Button_Closed.png");

        #region static methods

        public static void ForceRefresh()
        {
            m_Dockable_Window_Visible = false;
            InitCommandButton();
            ToggleIcon();
        }

        private static void InitCommandButton()
        {
            //Get a UID for the JambWow toolbar
            UID JambWowToolbar_UID = new UID();
            JambWowToolbar_UID.Value = "USACE_JAMBWOW_USACE_JAMBWOW";
            //Get a colletion of command bars
            IDocument ThisDoc = ArcMap.Application.Document;
            ICommandBars CommandBars = ThisDoc.CommandBars as ICommandBars;
            //Get the JambWow toolbar
            ICommandBar JambWowToolbar = CommandBars.Find(JambWowToolbar_UID, false, false) as ICommandBar;

            //Get a UID for the JambWow Hide/Show button
            UID JambWowHideShow_UID = new UID();                                        
            JambWowHideShow_UID.Value = "USACE_JAMBWOW_ShowJAMBWOW_Button";
            //Get the JambWow hide/show button
            m_HideShowButton = JambWowToolbar.Find(JambWowHideShow_UID, false);
        }

        private static void ToggleIcon()
        {
            //If the current dockable window state is the same as the previous state do nothing.
            IDockableWindow dockWindow = GetDockableWindow();

            bool bDocWindowVisible = dockWindow.IsVisible();
            if (m_Dockable_Window_Visible == bDocWindowVisible) return;
            if (m_HideShowButton == null) return;

            //If the docable window is open, set the open book icon.
            //If the docable window is closed, set the closed book icon.
            //System.Drawing.Bitmap newIcon;
            Bitmap pImage;
            if (bDocWindowVisible)
                pImage = m_OpenBookImage;
            else
                pImage = m_ClosedBookImage;
            m_HideShowButton.FaceID = ESRI.ArcGIS.ADF.COMSupport.OLE.GetIPictureDispFromBitmap(pImage);

            //Update the docable window saved state.
            m_Dockable_Window_Visible = bDocWindowVisible;

            //Allow the UI to refresh
            System.Windows.Forms.Application.DoEvents();
        }

        private static IDockableWindow GetDockableWindow()
        {
            UID dockWinID = new UIDClass();
            dockWinID.Value = ThisAddIn.IDs.JAMBWOW_DockableWindow;
            return ArcMap.DockableWindowManager.GetDockableWindow(dockWinID);
        }

        public static void ChangeJambWowIconOnWindowMenu()
        {
            //Get a UID for the Window Menu
            UID WindowMenu_UID = new UID();
            WindowMenu_UID.Value = "esriArcMapUI.MxWindowMenu";
            //Get a colletion of command bars
            IDocument ThisDoc = ArcMap.Application.Document;
            ICommandBars CommandBars = ThisDoc.CommandBars as ICommandBars;
            //Get the Window Menu
            ICommandBar WindowMenu = CommandBars.Find(WindowMenu_UID, false, false) as ICommandBar;

            //Get a UID for the JambWow Hide/Show button
            UID JambWowHideShow_UID = new UID();
            JambWowHideShow_UID.Value = "USACE_JAMBWOW_ShowJAMBWOW_Button";
            //Get the JambWow hide/show button
            ICommandItem pHideShowButton = WindowMenu.Find(JambWowHideShow_UID, false);

            //Set the icon to the open book image.
            Bitmap pImage = m_OpenBookImage;
            pHideShowButton.FaceID = ESRI.ArcGIS.ADF.COMSupport.OLE.GetIPictureDispFromBitmap(pImage);
        }

        #endregion

        #region instance methods

        public ShowJAMBWOW_Button()
        {
            if (m_singleton == null) m_singleton = this; //Initialize the singleton object.
            InitCommandButton();
        }

        protected override void OnClick()
        {
            try
            {               
                IDockableWindow dockWindow = GetDockableWindow();
                bool bWinVisible = dockWindow.IsVisible();
                base.Checked = bWinVisible;
                //Toggle the window visible/hidden.
                dockWindow.Show(!bWinVisible);                
                //Toggle the icon
                ToggleIcon();
            }
            catch (Exception pException)
            {
                GenUtil.ShowError(pException);
            }
        }
        protected override void OnUpdate()
        {
            try
            {
                Enabled = ArcMap.Application != null;
                IDockableWindow dockWindow = GetDockableWindow();
                bool bWinVisible = dockWindow.IsVisible();
                base.Checked = bWinVisible;
                ToggleIcon();
            }
            catch{}
        }

        #endregion
    }
}
