/*------------------------------------------------------------------
 *EGIS Gateway is an ArcMap extension which advertises available GIS data layers and map products.
 *Copyright (C) 2008  U.S. Army Corps of Engineers - New Orleans District
 *
 *This program is free software: you can redistribute it and/or modify
 *it under the terms of the GNU General Public License as published by
 *the Free Software Foundation, either version 3 of the License, or
 *(at your option) any later version.
 *
 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.
 *
 *You should have received a copy of the GNU General Public License
 *along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *Geospatial Engineering Section
 *USACE New Orleans District, Attn: ED-S
 *P.O. Box 60267
 *New Orleans, LA 70160-0267
 *------------------------------------------------------------------
 */

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Reflection;

namespace JAMBWOW
{
    public partial class aboutForm : Form
    {
        public aboutForm()
        {
            InitializeComponent();
            InitTextLabels();
        }

        private void btnOk_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void InitTextLabels()
        {
            //Get version number
            Assembly myAssembly = System.Reflection.Assembly.GetExecutingAssembly();
            AssemblyName myAssemblyName = myAssembly.GetName();
            Version myVersion = myAssemblyName.Version;
            string versionNum = myVersion.ToString();
            this.lblVersion.Text = "Version: " + versionNum;
            //Get Release date
            DateTime buildDate = RetrieveLinkerTimestamp();
            string strReleaseDT = buildDate.ToShortDateString();
            this.lblReleaseDate.Text = "Release Date: " + strReleaseDT;
        }

        //This function retrieves the timestamp of the last build
        private DateTime RetrieveLinkerTimestamp()
        {
            string filePath = System.Reflection.Assembly.GetCallingAssembly().Location;
            const int c_PeHeaderOffset = 60;
            const int c_LinkerTimestampOffset = 8;
            byte[] b = new byte[2048];
            System.IO.Stream s = null;

            try
            {
                s = new System.IO.FileStream(filePath, System.IO.FileMode.Open, System.IO.FileAccess.Read);
                s.Read(b, 0, 2048);
            }
            finally
            {
                if (s != null)
                {
                    s.Close();
                }
            }

            int i = System.BitConverter.ToInt32(b, c_PeHeaderOffset);
            int secondsSince1970 = System.BitConverter.ToInt32(b, i + c_LinkerTimestampOffset);
            DateTime dt = new DateTime(1970, 1, 1, 0, 0, 0);
            dt = dt.AddSeconds(secondsSince1970);
            dt = dt.AddHours(TimeZone.CurrentTimeZone.GetUtcOffset(dt).Hours);
            return dt;
        }
    }
}